package cz.uhk.fim.redakcnisystem.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserDetailsServiceImpl;
import io.jsonwebtoken.ExpiredJwtException;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Component
@RequiredArgsConstructor
public class JwtAuthFilter extends OncePerRequestFilter {

    private final UserDetailsServiceImpl userDetailsService;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        String header = request.getHeader(HttpHeaders.AUTHORIZATION);

        if (header != null) {
            String[] authElements = header.split(" ");
            if (authElements.length == 2 && "Bearer".equals(authElements[0])) {
                try {
                    if (JwtUtil.validateToken(authElements[1])) {
                        User user = (User) userDetailsService.loadUserByUsername(JwtUtil.extractUsername(authElements[1]));
                        SecurityContextHolder
                                .getContext()
                                .setAuthentication(
                                        new UsernamePasswordAuthenticationToken(user.getEmail(), "", user.getAuthorities()));
                    }
                } catch (Exception e){
                    SecurityContextHolder.clearContext();
                    if(e instanceof ExpiredJwtException) {
                        ObjectMapper mapper = new ObjectMapper();
                        response.setStatus(HttpStatus.UNAUTHORIZED.value());
                        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
                        response.setCharacterEncoding("UTF-8");
                        mapper.writeValue(response.getWriter(), new ServiceError("token-expired", "Přihlášení vypršelo."));
                        return;
                    }
                }
            }
        }
        filterChain.doFilter(request, response);

    }
}
