package cz.uhk.fim.redakcnisystem.config;

import io.jsonwebtoken.*;

import java.util.Base64;
import java.util.Date;

public class JwtUtil {

    private static final String SECRET = Base64.getEncoder().encodeToString("supertajnykod".getBytes());
    private static final long EXPIRATION_TIME = 1000 * 60 * 60; //ms * s * m = 6O min

    public static String generateToken(String username) {
        return Jwts.builder()
                .setSubject(username)
                .setExpiration(new Date(System.currentTimeMillis() + EXPIRATION_TIME))
                .signWith(SignatureAlgorithm.HS512, SECRET)
                .compact();
    }

    public static String extractUsernameFromHeader(String header){
        if(header.split(" ").length < 2)
            return null;
        String token = header.split(" ")[1];
        return extractUsername(token);
    }

    public static String extractUsername(String token){
        return Jwts.parser()
                .setSigningKey(SECRET)
                .parseClaimsJws(token)
                .getBody()
                .getSubject();
    }

    public static boolean validateTokenFromHeader(String header) throws ExpiredJwtException, UnsupportedJwtException, SignatureException {
        if(header == null)
            return false;
        if(header.split(" ").length < 2)
            return false;
        String token = header.split(" ")[1];
        Claims claims = Jwts.parser().setSigningKey(SECRET).parseClaimsJws(token).getBody();
        return !claims.isEmpty();
    }

    public static boolean validateToken(String token) throws ExpiredJwtException, UnsupportedJwtException, SignatureException {
        Claims claims = Jwts.parser().setSigningKey(SECRET).parseClaimsJws(token).getBody();
        return !claims.isEmpty();
    }

    public static String refreshTokenFromHeader(String tokenHeader) {
        String token = tokenHeader.split(" ")[1];
        Claims claims = Jwts.parser()
                .setSigningKey(SECRET)
                .parseClaimsJws(token)
                .getBody();
        claims.setExpiration(new Date(System.currentTimeMillis() + EXPIRATION_TIME));
        return Jwts.builder()
                .setClaims(claims)
                .signWith(SignatureAlgorithm.HS512, SECRET)
                .compact();
    }
}
