package cz.uhk.fim.redakcnisystem.contentmanagement.controller;

import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.AdminCommentFeedOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.AdminRatingFeedOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.AdminFeedService;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;

import java.time.ZonedDateTime;
import java.util.List;

@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping(path = "/api/v1/admin/content")
@RequiredArgsConstructor
@Secured("ROLE_ADMIN")
public class AdminContentController {

    private final AdminFeedService adminFeedService;

    @GetMapping("/feed/comment")
    public ResponseEntity<ServerResponse<List<AdminCommentFeedOut>>> getCommentsFeed(
            @RequestParam(name = "timeStart", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime timeStart,

            @RequestParam(name = "timeEnd", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime timeEnd){
        if(timeStart == null || timeEnd == null) {
            timeEnd = ZonedDateTime.now();
            timeStart = timeEnd.minusDays(1);
        }
        return ServerResponse.createResponseEntity(adminFeedService.getCommentsFeed(timeStart, timeEnd));
    }

    @GetMapping("/feed/rating")
    public ResponseEntity<ServerResponse<List<AdminRatingFeedOut>>> getRatingsFeed(
            @RequestParam(name = "timeStart", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime timeStart,

            @RequestParam(name = "timeEnd", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime timeEnd){
        if(timeStart == null || timeEnd == null) {
            timeEnd = ZonedDateTime.now();
            timeStart = timeEnd.minusDays(1);
        }
        return ServerResponse.createResponseEntity(adminFeedService.getRatingsFeed(timeStart, timeEnd));
    }
}
