package cz.uhk.fim.redakcnisystem.contentmanagement.controller;

import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.statistics.*;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.AdminStatisticsService;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;

import java.time.ZonedDateTime;
import java.util.List;

@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping(path = "/api/v1/admin/statistics")
@RequiredArgsConstructor
@Secured("ROLE_ADMIN")
public class AdminStatisticsController {

    private final AdminStatisticsService adminStatisticsService;

    @GetMapping("/article/views")
    public ResponseEntity<ServerResponse<List<AdminStatisticsPairOut>>> getArticlesViews(
            @RequestParam(value = "startDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime start,
            @RequestParam(value = "endDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime end) {
        if(start == null)
            start = ZonedDateTime.now().minusDays(7);
        if(end == null)
            end = ZonedDateTime.now();
        return ServerResponse.createResponseEntity(adminStatisticsService.getArticlesViewsBetweenDates(start, end));
    }

    @GetMapping("/article/comments")
    public ResponseEntity<ServerResponse<List<AdminStatisticsPairOut>>> getArticleComments(
            @RequestParam(value = "startDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime start,
            @RequestParam(value = "endDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime end) {
        if(start == null)
            start = ZonedDateTime.now().minusDays(7);
        if(end == null)
            end = ZonedDateTime.now();
        return ServerResponse.createResponseEntity(adminStatisticsService.getArticleCommentsBetweenDates(start, end));
    }

    @GetMapping("/article/ratings")
    public ResponseEntity<ServerResponse<List<AdminStatisticsPairOut>>> getArticlesRatings(
            @RequestParam(value = "startDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime start,
            @RequestParam(value = "endDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime end) {
        if(start == null)
            start = ZonedDateTime.now().minusDays(7);
        if(end == null)
            end = ZonedDateTime.now();
        return ServerResponse.createResponseEntity(adminStatisticsService.getArticleRatingsBetweenDates(start, end));
    }


    @GetMapping("/user/logins")
    public ResponseEntity<ServerResponse<List<AdminStatisticsPairOut>>> getUsersLogins(
            @RequestParam(value = "startDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime start,
            @RequestParam(value = "endDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime end,
            @RequestParam(value = "unique", required = false) Boolean unique) {
        if(start == null)
            start = ZonedDateTime.now().minusDays(7);
        if(end == null)
            end = ZonedDateTime.now();
        if(unique == null)
            unique = true;
        if(unique)
            return ServerResponse.createResponseEntity(adminStatisticsService.getUniqueUsersLoginsBetweenDates(start, end));
        return ServerResponse.createResponseEntity(adminStatisticsService.getUsersLoginsBetweenDates(start, end));
    }

    @GetMapping("/user/registrations")
    public ResponseEntity<ServerResponse<List<AdminStatisticsPairOut>>> getUsersRegistrations(
            @RequestParam(value = "startDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime start,
            @RequestParam(value = "endDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime end) {
        if (start == null)
            start = ZonedDateTime.now().minusDays(7);
        if (end == null)
            end = ZonedDateTime.now();
        return ServerResponse.createResponseEntity(adminStatisticsService.getUsersRegistrationsBetweenDates(start, end));
    }

    @GetMapping("/user/{userId}")
    public ResponseEntity<ServerResponse<AdminUserStatisticsOut>> getSpecificUserStats(
            @RequestParam(value = "startDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime start,
            @RequestParam(value = "endDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime end,
            @PathVariable String userId){
        if (start == null)
            start = ZonedDateTime.now().minusDays(7);
        if (end == null)
            end = ZonedDateTime.now();
        return ServerResponse.createResponseEntity(adminStatisticsService.getSpecificUserStats(start, end, Integer.valueOf(userId)));
    }

    @GetMapping("/user")
    public ResponseEntity<ServerResponse<List<AdminUserStatisticsSearchOut>>> findUserStatsOutList(
            @RequestParam("search") String usernameSearch){
        return ServerResponse.createResponseEntity(adminStatisticsService.searchUsersOut(usernameSearch));
    }

    @GetMapping("/article")
    public ResponseEntity<ServerResponse<List<AdminArticleStatisticsSearchOut>>> findArticleStatsOutList(
            @RequestParam("search") String articleSearch){
        return ServerResponse.createResponseEntity(adminStatisticsService.searchArticlesOut(articleSearch));
    }

    @GetMapping("/article/{articleId}")
    public ResponseEntity<ServerResponse<AdminArticleStatisticsOut>> getSpecificArticleStats(
            @RequestParam(value = "startDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime start,
            @RequestParam(value = "endDate", required = false)
            @DateTimeFormat(iso = DateTimeFormat.ISO.DATE_TIME) ZonedDateTime end,
            @PathVariable String articleId){
        if (start == null)
            start = ZonedDateTime.now().minusDays(7);
        if (end == null)
            end = ZonedDateTime.now();
        return ServerResponse.createResponseEntity(adminStatisticsService.getSpecificArticleStats(start, end, Integer.valueOf(articleId)));
    }
}
