package cz.uhk.fim.redakcnisystem.contentmanagement.controller;

import cz.uhk.fim.redakcnisystem.config.JwtUtil;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.ArticleSurveyVoteIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleSurveyOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.ArticleSurveyService;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;

@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping(path = "/api/v1/content")
@RequiredArgsConstructor
public class ArticleSurveyController {

    private final ArticleSurveyService articleSurveyService;
    private final UserService userService;

    @GetMapping("/article/{id}/survey")
    public ResponseEntity<ServerResponse<ArticleSurveyOut>> getSurveyForArticle(@PathVariable Integer id){
        return ServerResponse.createResponseEntity(articleSurveyService.getSurveyOutForArticle(id));
    }

    @Secured("ROLE_USER")
    @GetMapping("/article/{id}/survey/vote")
    public ResponseEntity<ServerResponse<Integer>> getSurveyVote(
            @PathVariable Integer id,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(articleSurveyService.getSurveyVote(id, userId));
    }

    @Secured("ROLE_USER")
    @PostMapping("/article/{id}/survey/vote")
    public ResponseEntity<ServerResponse<String>> postSurveyVote(
            @PathVariable Integer id,
            @RequestBody ArticleSurveyVoteIn voteIn,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(articleSurveyService.postSurveyVote(id, userId, voteIn));
    }
}
