package cz.uhk.fim.redakcnisystem.contentmanagement.controller;

import cz.uhk.fim.redakcnisystem.config.JwtUtil;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.CommentArticleIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.CommentArticleOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.CommentArticleService;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping(path = "/api/v1/content")
@RequiredArgsConstructor
public class CommentController {

    private final CommentArticleService commentArticleService;
    private final UserService userService;

    @GetMapping("/article/{id}/comments")
    public ResponseEntity<ServerResponse<List<CommentArticleOut>>> getArticleComments(@PathVariable String id){
        return ServerResponse.createResponseEntity(commentArticleService.getArticleComments(Integer.valueOf(id)));
    }

    @Secured("ROLE_USER")
    @PostMapping("/article/{id}/comment")
    public ResponseEntity<ServerResponse<String>> addCommentToArticle(
            @PathVariable String id,
            @RequestBody CommentArticleIn commentArticleIn,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        commentArticleIn.setArticleId(Integer.parseInt(id));
        return ServerResponse.createResponseEntity(commentArticleService.addCommentToArticle(commentArticleIn, userId));
    }

    @Secured("ROLE_USER")
    @DeleteMapping("/article/comment/{commentId}")
    public ResponseEntity<ServerResponse<String>> deleteCommentArticle(
            @PathVariable("commentId") Long id,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(commentArticleService.deleteComment(id, userId));
    }

    @Secured("ROLE_ADMIN")
    @PutMapping("/article/comment/{commentId}/incorrect")
    public ResponseEntity<ServerResponse<String>> setCommentIncorrect(@PathVariable Long commentId, @RequestBody Boolean incorrect){
        return ServerResponse.createResponseEntity(commentArticleService.setCommentIncorrect(commentId, incorrect));
    }
}
