package cz.uhk.fim.redakcnisystem.contentmanagement.controller;

import cz.uhk.fim.redakcnisystem.config.JwtUtil;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.ArticleRatingIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleRatingOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleRatingUserOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.RatingArticleService;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;

@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping(path = "/api/v1/content")
@RequiredArgsConstructor
public class RatingController {

    private final RatingArticleService ratingArticleService;
    private final UserService userService;

    @GetMapping("/article/{id}/rating")
    public ResponseEntity<ServerResponse<ArticleRatingOut>> getRatingForArticle(@PathVariable String id){
        return ServerResponse.createResponseEntity(ratingArticleService.getRatingForArticle(Integer.valueOf(id)));
    }

    @GetMapping("/article/{id}/rating/user")
    public ResponseEntity<ServerResponse<ArticleRatingUserOut>> getRatingForArticleUser(
            @PathVariable String id,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){

        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(
                ratingArticleService.getRatingForArticleUser(Integer.valueOf(id), userId));
    }

    @Secured("ROLE_USER")
    @PostMapping("/article/{id}/rating")
    public ResponseEntity<ServerResponse<String>> addRatingToArticle(
            @PathVariable String id,
            @RequestBody ArticleRatingIn articleRatingIn,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){

        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        articleRatingIn.setArticleId(Integer.parseInt(id));
        return ServerResponse.createResponseEntity(ratingArticleService.addRating(articleRatingIn, userId));
    }

    @Secured("ROLE_USER")
    @DeleteMapping("/article/{id}/rating/user")
    public ResponseEntity<ServerResponse<String>> deleteRatingForArticleUser(
            @PathVariable String id,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer userId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        return ServerResponse.createResponseEntity(ratingArticleService.deleteRatingForUser(Integer.valueOf(id), userId));
    }
}
