package cz.uhk.fim.redakcnisystem.contentmanagement.controller;

import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.GalleryImageInfoOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.json.AuthorPageData;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.json.GeneralSettings;
import cz.uhk.fim.redakcnisystem.contentmanagement.service.SettingsService;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping(path = "/api/v1/settings")
@RequiredArgsConstructor
public class SettingsController {

    private final SettingsService settingsService;

    @GetMapping(value = "/favicon.ico", produces = "image/x-icon")
    public byte[] getFavicon() throws IOException {
        return settingsService.getFavicon();
    }

    @Secured("ROLE_ADMIN")
    @PostMapping(value = "/favicon.ico", consumes = "image/x-icon")
    public ResponseEntity<ServerResponse<String>> saveFavicon(@RequestBody byte[] icon) throws IOException {
        return ServerResponse.createResponseEntity(settingsService.updateFavicon(icon));
    }

    @GetMapping("/general")
    public ResponseEntity<ServerResponse<GeneralSettings>> getGeneralSettings() throws IOException {
        return ServerResponse.createResponseEntity(settingsService.getGeneralSettings());
    }

    @Secured("ROLE_ADMIN")
    @PostMapping("/general")
    public ResponseEntity<ServerResponse<String>> saveGeneralSettings(@RequestBody GeneralSettings generalSettings){
        return ServerResponse.createResponseEntity(settingsService.updateGeneralSettings(generalSettings));
    }

    @GetMapping("/author-page")
    public ResponseEntity<ServerResponse<AuthorPageData>> getAuthorPageData() throws IOException {
        return ServerResponse.createResponseEntity(settingsService.getAuthorPageData());
    }

    @Secured("ROLE_ADMIN")
    @PostMapping("/author-page")
    public ResponseEntity<ServerResponse<String>> updateAuthorPageData(@RequestBody AuthorPageData authorPageData){
        return ServerResponse.createResponseEntity(settingsService.updateAuthorPageData(authorPageData));
    }

    @Secured("ROLE_ADMIN")
    @PostMapping("/gallery/upload")
    public ResponseEntity<ServerResponse<String>> uploadImages(@RequestParam("files")MultipartFile[] files) throws IOException {
        return ServerResponse.createResponseEntity(settingsService.uploadImages(files));
    }

    @GetMapping("/gallery/image/{imageName}")
    public ResponseEntity<byte[]> getImage(@PathVariable String imageName) throws IOException {
        return settingsService.getImage(imageName);
    }

    @GetMapping("/gallery/image/list")
    public ResponseEntity<ServerResponse<List<GalleryImageInfoOut>>> getListOfAllImages(){
        return ServerResponse.createResponseEntity(settingsService.getAllImagesNames());
    }

    @Secured("ROLE_ADMIN")
    @DeleteMapping("/gallery/image/{imageName}")
    public ResponseEntity<ServerResponse<String>> deleteImage(@PathVariable String imageName){
        return ServerResponse.createResponseEntity(settingsService.deleteImage(imageName));
    }
}
