package cz.uhk.fim.redakcnisystem.contentmanagement.records;

import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.ArticleIn;
import cz.uhk.fim.redakcnisystem.usermanagement.records.UserSavedArticle;
import lombok.Data;

import javax.persistence.*;
import java.time.ZonedDateTime;
import java.util.List;

@Data
@Entity
public class Article {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(columnDefinition = "int unsigned")
    private Integer id;

    @Lob
    private String text;
    private String title;
    private String description;
    private Boolean visibility;
    private ZonedDateTime createTime = ZonedDateTime.now();
    private ZonedDateTime lastUpdateTime = ZonedDateTime.now();

    @OneToMany(mappedBy = "article", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<RatingArticle> ratings;

    @OneToMany(mappedBy = "article", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<CommentArticle> comments;

    @OneToMany(mappedBy = "article", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<UserSavedArticle> usersSavedArticle;

    @OneToMany(mappedBy = "article", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<ArticleViewActivity> articleViewActivities;

    @OneToOne(mappedBy = "article", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private ArticleSurvey articleSurvey;

    public void mapDataFromArticleIn(ArticleIn articleIn){
        setTitle(articleIn.getTitle());
        setText(articleIn.getText());
        setVisibility(articleIn.getVisibility());
        setDescription(articleIn.getDescription());
    }
}
