package cz.uhk.fim.redakcnisystem.contentmanagement.repository;

import cz.uhk.fim.redakcnisystem.contentmanagement.records.Article;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.RatingArticle;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;

import java.time.ZonedDateTime;
import java.util.List;
import java.util.Optional;

public interface RatingArticleRepository extends CrudRepository<RatingArticle, Long> {
    Optional<RatingArticle> findByUserIdAndArticleId(Integer userId, Integer articleId);
    List<RatingArticle> findAllByArticleId(Integer articleId);
    List<RatingArticle> findAllByUserId(Integer userId);
    Integer countAllByUserId(Integer userId);

    List<RatingArticle> findAllByCreateTimeIsBetween(ZonedDateTime start, ZonedDateTime end);

    Optional<Integer> countAllByCreateTimeBetween(ZonedDateTime createTimeStart, ZonedDateTime createTimeEnd);

    @Query("SELECT avg(r.rating) FROM RatingArticle r WHERE r.user = :user")
    Float getRatingAverageByUser(User user);

    Optional<Integer> countAllByCreateTimeBetweenAndUserId(ZonedDateTime createTimeStart, ZonedDateTime createTimeEnd, Integer user_id);

    Optional<Integer> countAllByArticleIdAndRatingEquals(Integer articleId, Integer rating);

    Optional<Integer> countAllByCreateTimeBetweenAndArticleId(ZonedDateTime createTimeStart, ZonedDateTime createTimeEnd, Integer article_id);

    Optional<Integer> countAllByCreateTimeBeforeAndArticleId(ZonedDateTime createTime, Integer article_id);

    @Query("SELECT avg(r.rating) FROM RatingArticle r WHERE r.article = :article")
    Optional<Float> getAverageRatingByArticle(Article article);
}
