package cz.uhk.fim.redakcnisystem.contentmanagement.service;

import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.AdminCommentFeedOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.AdminRatingFeedOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.CommentArticle;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.CommentArticleRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.RatingArticleRepository;
import cz.uhk.fim.redakcnisystem.response.ServiceError;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;

@Service
@RequiredArgsConstructor
public class AdminFeedServiceImpl implements AdminFeedService{

    private final CommentArticleRepository commentRepository;
    private final RatingArticleRepository ratingRepository;

    @Override
    public ServiceResult<List<AdminCommentFeedOut>> getCommentsFeed(ZonedDateTime timeStart, ZonedDateTime timeEnd) {
        List<AdminCommentFeedOut> commentFeedOuts = new ArrayList<>();
        List<CommentArticle> comments = commentRepository.findAllByCreateTimeIsBetween(timeStart, timeEnd);
        comments.forEach(commentArticle -> {
            AdminCommentFeedOut commentFeedTemp = new AdminCommentFeedOut();
            commentFeedTemp.setCommentData(commentArticle);
            commentFeedTemp.setUserData(commentArticle.getUser());
            commentFeedTemp.setArticleData(commentArticle.getArticle());

            commentFeedOuts.add(commentFeedTemp);
        });
        if(commentFeedOuts.isEmpty())
            return ServiceResult.createServiceResult(
                    List.of(
                            new ServiceError("comment-feed-not-found","Komentáře za dané období nebyly nalezeny.")),
                    HttpStatus.NO_CONTENT);
        return ServiceResult.createServiceResult(commentFeedOuts, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<AdminRatingFeedOut>> getRatingsFeed(ZonedDateTime timeStart, ZonedDateTime timeEnd) {
        List<AdminRatingFeedOut> ratingFeedOuts = new ArrayList<>();
        ratingRepository.findAllByCreateTimeIsBetween(timeStart, timeEnd).forEach(ratingArticle -> {
            AdminRatingFeedOut ratingFeedTemp = new AdminRatingFeedOut();
            ratingFeedTemp.setFromRating(ratingArticle);
            ratingFeedOuts.add(ratingFeedTemp);
        });
        if(ratingFeedOuts.isEmpty()){
            return ServiceResult.createServiceResult(
                    List.of(
                            new ServiceError("rating-feed-not-found", "Hodnocení za dané období nebyly nalezeny.")),
                    HttpStatus.NO_CONTENT);
        }
        return ServiceResult.createServiceResult(ratingFeedOuts, HttpStatus.OK);
    }
}
