package cz.uhk.fim.redakcnisystem.contentmanagement.service;

import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.statistics.*;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.Article;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.*;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserLoginActivityRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;


import java.time.LocalDate;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class AdminStatisticsServiceImpl implements AdminStatisticsService{

    private final ArticleViewActivityRepository articleViewActivityRepository;
    private final UserLoginActivityRepository userLoginActivityRepository;
    private final UserRepository userRepository;
    private final ArticleRepository articleRepository;
    private final CommentArticleRepository commentArticleRepository;
    private final RatingArticleRepository ratingArticleRepository;
    private final ArticleSurveyVoteRepository articleSurveyVoteRepository;
    private final ArticleSurveyOptionRepository articleSurveyOptionRepository;

    @Override
    public ServiceResult<List<AdminStatisticsPairOut>> getArticlesViewsBetweenDates(ZonedDateTime start, ZonedDateTime end) {
        List<AdminStatisticsPairOut> out = new ArrayList<>();
        ZonedDateTime startDate = start.toLocalDate().atStartOfDay(start.getZone());
        ZonedDateTime endDate = startDate.toLocalDate().atStartOfDay(start.getZone()).plusDays(1);
        for (int i = 0; i <= daysBetweenTwoDates(start, end); i++) {
            AdminStatisticsPairOut adminArticleViewsOut = new AdminStatisticsPairOut();
            adminArticleViewsOut.setDate(startDate);
            Optional<Integer> count = articleViewActivityRepository.countAllByArticleViewTimeBetween(startDate, endDate);
            adminArticleViewsOut.setCount(count.orElse(0));
            out.add(adminArticleViewsOut);
            startDate = startDate.plusDays(1);
            endDate = endDate.plusDays(1);
        }
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<AdminStatisticsPairOut>> getArticleCommentsBetweenDates(ZonedDateTime start, ZonedDateTime end) {
        List<AdminStatisticsPairOut> out = new ArrayList<>();
        ZonedDateTime startDate = start.toLocalDate().atStartOfDay(start.getZone());
        ZonedDateTime endDate = startDate.toLocalDate().atStartOfDay(start.getZone()).plusDays(1);
        for (int i = 0; i <= daysBetweenTwoDates(start, end); i++) {
            AdminStatisticsPairOut adminArticleViewsOut = new AdminStatisticsPairOut();
            adminArticleViewsOut.setDate(startDate);
            Optional<Integer> count = commentArticleRepository.countAllByCreateTimeBetween(startDate, endDate);
            adminArticleViewsOut.setCount(count.orElse(0));
            out.add(adminArticleViewsOut);
            startDate = startDate.plusDays(1);
            endDate = endDate.plusDays(1);
        }
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<AdminStatisticsPairOut>> getArticleRatingsBetweenDates(ZonedDateTime start, ZonedDateTime end) {
        List<AdminStatisticsPairOut> out = new ArrayList<>();
        ZonedDateTime startDate = start.toLocalDate().atStartOfDay(start.getZone());
        ZonedDateTime endDate = startDate.toLocalDate().atStartOfDay(start.getZone()).plusDays(1);
        for (int i = 0; i <= daysBetweenTwoDates(start, end); i++) {
            AdminStatisticsPairOut adminArticleViewsOut = new AdminStatisticsPairOut();
            adminArticleViewsOut.setDate(startDate);
            Optional<Integer> count = ratingArticleRepository.countAllByCreateTimeBetween(startDate, endDate);
            adminArticleViewsOut.setCount(count.orElse(0));
            out.add(adminArticleViewsOut);
            startDate = startDate.plusDays(1);
            endDate = endDate.plusDays(1);
        }
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<AdminStatisticsPairOut>> getUniqueUsersLoginsBetweenDates(ZonedDateTime start, ZonedDateTime end) {
        List<AdminStatisticsPairOut> out = new ArrayList<>();
        ZonedDateTime startDate = start.toLocalDate().atStartOfDay(start.getZone());
        ZonedDateTime endDate = startDate.toLocalDate().atStartOfDay(start.getZone()).plusDays(1);
        for (int i = 0; i <= daysBetweenTwoDates(start, end); i++) {
            AdminStatisticsPairOut adminArticleViewsOut = new AdminStatisticsPairOut();
            adminArticleViewsOut.setDate(startDate);
            Optional<Integer> count = userLoginActivityRepository.countUniqueUserBetweenTimes(startDate, endDate);
            adminArticleViewsOut.setCount(count.orElse(0));
            out.add(adminArticleViewsOut);
            startDate = startDate.plusDays(1);
            endDate = endDate.plusDays(1);
        }
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<AdminStatisticsPairOut>> getUsersLoginsBetweenDates(ZonedDateTime start, ZonedDateTime end) {
        List<AdminStatisticsPairOut> out = new ArrayList<>();
        ZonedDateTime startDate = start.toLocalDate().atStartOfDay(start.getZone());
        ZonedDateTime endDate = startDate.toLocalDate().atStartOfDay(start.getZone()).plusDays(1);
        for (int i = 0; i <= daysBetweenTwoDates(start, end); i++) {
            AdminStatisticsPairOut adminArticleViewsOut = new AdminStatisticsPairOut();
            adminArticleViewsOut.setDate(startDate);
            Optional<Integer> count = userLoginActivityRepository.countAllByLoginTimeBetween(startDate, endDate);
            adminArticleViewsOut.setCount(count.orElse(0));
            out.add(adminArticleViewsOut);
            startDate = startDate.plusDays(1);
            endDate = endDate.plusDays(1);
        }
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<AdminStatisticsPairOut>> getUsersRegistrationsBetweenDates(ZonedDateTime start, ZonedDateTime end) {
        List<AdminStatisticsPairOut> out = new ArrayList<>();
        ZonedDateTime startDate = start.toLocalDate().atStartOfDay(start.getZone());
        ZonedDateTime endDate = startDate.toLocalDate().atStartOfDay(start.getZone()).plusDays(1);
        for (int i = 0; i <= daysBetweenTwoDates(start, end); i++) {
            AdminStatisticsPairOut adminArticleViewsOut = new AdminStatisticsPairOut();
            adminArticleViewsOut.setDate(startDate);
            Optional<Integer> count = userRepository.countAllByCreateTimeBetween(startDate, endDate);
            adminArticleViewsOut.setCount(count.orElse(0));
            out.add(adminArticleViewsOut);
            startDate = startDate.plusDays(1);
            endDate = endDate.plusDays(1);
        }
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<AdminUserStatisticsOut> getSpecificUserStats(ZonedDateTime start, ZonedDateTime end, Integer userId) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        AdminUserStatisticsOut userStatisticsOut = new AdminUserStatisticsOut();
        if(userOptional.get().getBirthDate() != null)
            userStatisticsOut.setAge(
                    LocalDate.now().getYear() -
                            userOptional.get().getBirthDate().toInstant().atZone(ZoneId.systemDefault()).getYear());
        userStatisticsOut.setCountAllComments(commentArticleRepository.countAllByUserId(userId));
        userStatisticsOut.setCountAllRatings(ratingArticleRepository.countAllByUserId(userId));
        userStatisticsOut.setAverageOfAllRatings(ratingArticleRepository.getRatingAverageByUser(userOptional.get()));
        userStatisticsOut.setOtherInfoFromUser(userOptional.get());

        ZonedDateTime startDate = start.toLocalDate().atStartOfDay(start.getZone());
        ZonedDateTime endDate = startDate.toLocalDate().atStartOfDay(start.getZone()).plusDays(1);
        for (int i = 0; i <=daysBetweenTwoDates(start, end); i++) {
            userStatisticsOut.addArticleRatings(
                    new AdminStatisticsPairOut(
                            startDate,
                            ratingArticleRepository.countAllByCreateTimeBetweenAndUserId(startDate, endDate, userId).orElse(0)));
            userStatisticsOut.addArticleComments(
                    new AdminStatisticsPairOut(
                            startDate,
                            commentArticleRepository.countAllByCreateTimeBetweenAndUserId(startDate, endDate, userId).orElse(0)));
            userStatisticsOut.addArticleReads(
                    new AdminStatisticsPairOut(
                            startDate,
                            articleViewActivityRepository.countAllByArticleViewTimeBetweenAndUserId(startDate, endDate, userId).orElse(0)));
            userStatisticsOut.addUniqueLogins(
                    new AdminStatisticsPairOut(
                            startDate,
                            userLoginActivityRepository.countAllByLoginTimeBetweenAndUserId(startDate, endDate, userId).orElse(0)));
            startDate = startDate.plusDays(1);
            endDate = endDate.plusDays(1);
        }

        return ServiceResult.createServiceResult(userStatisticsOut, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<AdminUserStatisticsSearchOut>> searchUsersOut(String usernameSearch) {
        List<AdminUserStatisticsSearchOut> out = new ArrayList<>();
        userRepository.findAllByUsernameContainsOrUsernameLike(usernameSearch,usernameSearch).forEach(user -> out.add(new AdminUserStatisticsSearchOut(user.getId(), user.getUsername())));
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<List<AdminArticleStatisticsSearchOut>> searchArticlesOut(String articleSearch) {
        List<AdminArticleStatisticsSearchOut> out = new ArrayList<>();
        articleRepository.findAllByTitleLikeOrTitleContains(articleSearch, articleSearch).forEach(article -> out.add(new AdminArticleStatisticsSearchOut(article.getId(), article.getTitle())));
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<AdminArticleStatisticsOut> getSpecificArticleStats(ZonedDateTime start, ZonedDateTime end, Integer articleId) {
        Optional<Article> articleOptional = articleRepository.findById(articleId);
        if(articleOptional.isEmpty())
            return Errors.articleNotFound();
        AdminArticleStatisticsOut articleStatisticsOut = new AdminArticleStatisticsOut();
        articleStatisticsOut.saveOtherInfoFromArticle(articleOptional.get());
        articleStatisticsOut.setRating(ratingArticleRepository.getAverageRatingByArticle(articleOptional.get()).orElse(0F));
        for (int i = 0; i < articleStatisticsOut.getSumOfEachRatingValue().length; i++) {
            articleStatisticsOut.saveValueForSumOfEachRatingByIndex(
                    ratingArticleRepository.countAllByArticleIdAndRatingEquals(articleId, (i+1)).orElse(0),
                    i);
        }
        if(articleOptional.get().getArticleSurvey() != null) {
            articleStatisticsOut.setSurvey(new AdminSurveyStatisticsOut());
            articleStatisticsOut.getSurvey().setSurveyText(articleOptional.get().getArticleSurvey().getSurveyText());
            articleSurveyOptionRepository.findAllByArticleSurveyId(
                    articleOptional.get().getArticleSurvey().getId()).forEach(articleSurveyOption -> articleStatisticsOut.getSurvey().addSurveyOptionsWithVotes(
                            new AdminStatisticsSurveyOptionVotesOut(
                                    articleSurveyOption.getPosition(),
                                    articleSurveyOption.getText(),
                                    articleSurveyVoteRepository
                                            .countAllByArticleSurveyIdAndSurveyOptionId(
                                                    articleOptional.get().getArticleSurvey().getId(),
                                                    articleSurveyOption.getId())
                                            .orElse(0))));
            articleStatisticsOut.getSurvey().setCountOfAllVotes(
                    articleSurveyVoteRepository.countAllByArticleSurveyId(
                            articleOptional.get().getArticleSurvey().getId()).orElse(0));
        }
        ZonedDateTime startDate = start.toLocalDate().atStartOfDay(start.getZone());
        ZonedDateTime endDate = startDate.toLocalDate().atStartOfDay(start.getZone()).plusDays(1);
        for (int i = 0; i <=daysBetweenTwoDates(start, end); i++) {
            articleStatisticsOut.addNewArticleRatings(new AdminStatisticsPairOut(
                    startDate,
                    ratingArticleRepository.countAllByCreateTimeBetweenAndArticleId(startDate, endDate, articleId).orElse(0)));
            articleStatisticsOut.addNewArticleComments(new AdminStatisticsPairOut(
                    startDate,
                    commentArticleRepository.countAllByCreateTimeBetweenAndArticleId(startDate, endDate, articleId).orElse(0)));
            articleStatisticsOut.addNewArticleViews(new AdminStatisticsPairOut(
                    startDate,
                    articleViewActivityRepository.countAllByArticleViewTimeBetweenAndArticleId(startDate, endDate, articleId).orElse(0)));

            articleStatisticsOut.addAllArticleRatings(new AdminStatisticsPairOut(
                    startDate,
                    ratingArticleRepository.countAllByCreateTimeBeforeAndArticleId(endDate, articleId).orElse(0)));
            articleStatisticsOut.addAllArticleComments(new AdminStatisticsPairOut(
                    startDate,
                    commentArticleRepository.countAllByCreateTimeBeforeAndArticleId(endDate, articleId).orElse(0)));
            articleStatisticsOut.addAllArticleViews(new AdminStatisticsPairOut(
                    startDate,
                    articleViewActivityRepository.countAllByArticleViewTimeBeforeAndArticleId(endDate, articleId).orElse(0)));
            if(articleStatisticsOut.getSurvey() != null) {
                articleStatisticsOut.getSurvey().addVotesPerDay(new AdminStatisticsPairOut(
                        startDate,
                        articleSurveyVoteRepository.countAllByCreateTimeBetweenAndArticleSurveyId(
                                startDate,
                                endDate,
                                articleOptional.get().getArticleSurvey().getId()).orElse(0)));
            }
            startDate = startDate.plusDays(1);
            endDate = endDate.plusDays(1);
        }
        return ServiceResult.createServiceResult(articleStatisticsOut, HttpStatus.OK);
    }

    private Integer daysBetweenTwoDates(ZonedDateTime start, ZonedDateTime end){
        return Math.toIntExact(ChronoUnit.DAYS.between(start, end))+1;
    }
}
