package cz.uhk.fim.redakcnisystem.contentmanagement.service;

import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoIn.ArticleSurveyVoteIn;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleSurveyOptionOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.dtoOut.ArticleSurveyOut;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.ArticleSurvey;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.ArticleSurveyOption;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.ArticleSurveyVote;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.ArticleSurveyOptionRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.ArticleSurveyRepository;
import cz.uhk.fim.redakcnisystem.contentmanagement.repository.ArticleSurveyVoteRepository;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserRepository;
import lombok.RequiredArgsConstructor;
import org.modelmapper.ModelMapper;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class ArticleSurveyServiceImpl implements ArticleSurveyService{

    private final ArticleSurveyRepository articleSurveyRepository;
    private final ArticleSurveyOptionRepository articleSurveyOptionRepository;
    private final ArticleSurveyVoteRepository articleSurveyVoteRepository;
    private final UserRepository userRepository;
    private final ModelMapper mapper = new ModelMapper();

    @Override
    public ServiceResult<ArticleSurveyOut> getSurveyOutForArticle(Integer articleId) {
        Optional<ArticleSurvey> articleSurveyOptional = articleSurveyRepository.findByArticleId(articleId);
        if(articleSurveyOptional.isEmpty())
            return Errors.articleHasNotSurvey();
        ArticleSurveyOut out = mapper.map(articleSurveyOptional.get(), ArticleSurveyOut.class);
        if(out.getSurveyVisibleResults()){
            List<ArticleSurveyOptionOut> options = new ArrayList<>();
            articleSurveyOptional.get().getSurveyOptions().forEach(option -> {
                ArticleSurveyOptionOut tempOut = new ArticleSurveyOptionOut();
                tempOut.setText(option.getText());
                tempOut.setPosition(option.getPosition());
                tempOut.setCount(articleSurveyVoteRepository.countAllByArticleSurveyIdAndSurveyOptionId(
                        articleSurveyOptional.get().getId(), option.getId()).orElse(0));
                options.add(tempOut);
            });
            out.setSurveyOptions(options);
        }
        return ServiceResult.createServiceResult(out, HttpStatus.OK);
    }

    @Override
    public ServiceResult<String> postSurveyVote(Integer articleId, Integer userId, ArticleSurveyVoteIn voteIn) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        Optional<ArticleSurvey> articleSurveyOptional = articleSurveyRepository.findByArticleId(articleId);
        if(articleSurveyOptional.isEmpty())
            return Errors.articleHasNotSurvey();
        Optional<ArticleSurveyOption> surveyOptionOptional = articleSurveyOptionRepository
                .findByArticleSurveyIdAndPosition(articleSurveyOptional.get().getId(), voteIn.getPosition());
        if(surveyOptionOptional.isEmpty())
            return Errors.singletonError(
                    "survey-has-not-selected-option",
                    "Anketa neobsahuje zvolenou možnost.",
                    HttpStatus.NOT_ACCEPTABLE);
        Optional<ArticleSurveyVote> articleSurveyVoteOptional = articleSurveyVoteRepository
                .findByArticleSurveyIdAndUserId(articleSurveyOptional.get().getId(), userId);
        ArticleSurveyVote articleSurveyVote;
        if(articleSurveyVoteOptional.isEmpty()){
            articleSurveyVote = new ArticleSurveyVote();
            articleSurveyVote.setSurveyOption(surveyOptionOptional.get());
            articleSurveyVote.setArticleSurvey(articleSurveyOptional.get());
            articleSurveyVote.setUser(userOptional.get());
        } else {
            articleSurveyVote = articleSurveyVoteOptional.get();
            articleSurveyVote.setCreateTime(ZonedDateTime.now());
            articleSurveyVote.setSurveyOption(surveyOptionOptional.get());
        }
        articleSurveyVoteRepository.save(articleSurveyVote);
        return ServiceResult.createServiceResult("Hlasování bylo úspěšné.", HttpStatus.OK);
    }

    @Override
    public ServiceResult<Integer> getSurveyVote(Integer articleId, Integer userId) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        Optional<ArticleSurvey> articleSurveyOptional = articleSurveyRepository.findByArticleId(articleId);
        if(articleSurveyOptional.isEmpty())
            return Errors.articleHasNotSurvey();
        Optional<ArticleSurveyVote> surveyVoteOptional = articleSurveyVoteRepository
                .findByArticleSurveyIdAndUserId(articleSurveyOptional.get().getId(), userId);
        if(surveyVoteOptional.isPresent())
            return ServiceResult.createServiceResult(surveyVoteOptional.get().getSurveyOption().getPosition(), HttpStatus.OK);
        return Errors.singletonError(
                "user-has-not-vote-in-this-survey",
                "Uživatel v této anketě nehlasoval.",
                HttpStatus.NOT_FOUND);
    }
}
