package cz.uhk.fim.redakcnisystem.usermanagement.controller;

import cz.uhk.fim.redakcnisystem.config.JwtUtil;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserAdminOut;
import cz.uhk.fim.redakcnisystem.usermanagement.records.enums.UserRoleEn;
import cz.uhk.fim.redakcnisystem.usermanagement.service.AdminUserService;
import cz.uhk.fim.redakcnisystem.usermanagement.service.UserService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequestMapping(path = "/api/v1/admin/user-management")
@RequiredArgsConstructor
@Secured("ROLE_ADMIN")
public class AdminUserController {

    private final UserService userService;
    private final AdminUserService adminUserService;

    @PutMapping("/user/{id}/role")
    public ResponseEntity<ServerResponse<String>> editUserRoles(
            @PathVariable Integer id,
            @RequestBody List<UserRoleEn> roles,
            @RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader){
        Integer loggedId = userService.getUserId(JwtUtil.extractUsernameFromHeader(tokenHeader));
        if(Objects.equals(id, loggedId))
            return null;
        return ServerResponse.createResponseEntity(adminUserService.editUserRoles(id, roles));
    }

    @Secured("ROLE_ADMIN")
    @GetMapping("/user")
    public ResponseEntity<ServerResponse<List<UserAdminOut>>> getUsersListForAdmin(){
        return ServerResponse.createResponseEntity(userService.getUsersListAdmin());
    }
}
