package cz.uhk.fim.redakcnisystem.usermanagement.controller;

import cz.uhk.fim.redakcnisystem.config.JwtUtil;
import cz.uhk.fim.redakcnisystem.response.ServerResponse;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserLoginIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoIn.UserRegistrationIn;
import cz.uhk.fim.redakcnisystem.usermanagement.dtoOut.UserTokenOut;
import cz.uhk.fim.redakcnisystem.usermanagement.service.AuthService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;


@CrossOrigin(origins = "http://localhost:3000", allowCredentials = "true")
@RestController
@RequiredArgsConstructor
@RequestMapping("api/v1/auth")
public class AuthController {

    private final AuthService authService;

    @PostMapping(value = "/registration", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ResponseEntity<ServerResponse<String>> registration(
            @RequestPart(name = "userData") UserRegistrationIn userRegistrationIn,
            @RequestPart(name = "image", required = false) MultipartFile image) {
        return ServerResponse.createResponseEntity(authService.registration(userRegistrationIn, image));
    }

    @PostMapping("/login")
    public ResponseEntity<ServerResponse<UserTokenOut>> login(@RequestBody UserLoginIn userLoginIn) {
        ServiceResult<UserTokenOut> result = authService.login(userLoginIn);
        return ServerResponse.createResponseEntity(result);
    }

    @Secured({"ROLE_USER"})
    @GetMapping("/token")
    public ResponseEntity<ServerResponse<UserTokenOut>> token(@RequestHeader(HttpHeaders.AUTHORIZATION) String tokenHeader) {
        UserTokenOut userTokenOut = UserTokenOut.builder()
                .token(JwtUtil.refreshTokenFromHeader(tokenHeader))
                .username(JwtUtil.extractUsernameFromHeader(tokenHeader))
                .build();
        return ServerResponse.createResponseEntity(ServiceResult.createServiceResult(userTokenOut, HttpStatus.OK));
    }

}
