package cz.uhk.fim.redakcnisystem.usermanagement.records;

import cz.uhk.fim.redakcnisystem.contentmanagement.records.ArticleSurveyVote;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.ArticleViewActivity;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.CommentArticle;
import cz.uhk.fim.redakcnisystem.contentmanagement.records.RatingArticle;
import cz.uhk.fim.redakcnisystem.usermanagement.records.enums.UserGenderEn;
import lombok.Data;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import javax.persistence.*;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

@Data
@Entity
@Table(name = "user")
public class User implements UserDetails {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(columnDefinition = "int unsigned")
    private Integer id;
    @Column(length = 33)
    private String username;
    private String email;
    private String password;

    @Column(name = "birth_date", columnDefinition = "date")
    private Date birthDate;

    @Column(length = 300)
    private String bio;

    @Enumerated(EnumType.STRING)
    private UserGenderEn gender;

    @Column(name = "create_time")
    private ZonedDateTime createTime = ZonedDateTime.now();

    @OneToMany(mappedBy = "user", fetch = FetchType.EAGER, cascade = CascadeType.REMOVE)
    private List<UserRole> roles;

    @OneToOne(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private UserImage userImage;

    @OneToOne(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private UserPrivacySettings userPrivacySettings;

    @OneToMany(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<RatingArticle> ratingArticles;

    @OneToMany(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<CommentArticle> commentArticles;

    @OneToMany(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<UserSavedArticle> savedArticles;

    @OneToMany(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<UserLoginActivity> userLoginActivities;

    @OneToMany(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.DETACH)
    private List<ArticleViewActivity> articleViewActivities;

    @OneToMany(mappedBy = "user", fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    private List<ArticleSurveyVote> articleSurveyVotes;

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        List<SimpleGrantedAuthority> authorities = new ArrayList<>();
        getRoles().forEach(userRole ->
                authorities.add(
                        new SimpleGrantedAuthority("ROLE_" + userRole.getRole().name())));
        return authorities;
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    public boolean isEnabled() {
        return true;
    }
}