package cz.uhk.fim.redakcnisystem.usermanagement.service;

import cz.uhk.fim.redakcnisystem.constants.Errors;
import cz.uhk.fim.redakcnisystem.response.ServiceResult;
import cz.uhk.fim.redakcnisystem.usermanagement.records.User;
import cz.uhk.fim.redakcnisystem.usermanagement.records.UserRole;
import cz.uhk.fim.redakcnisystem.usermanagement.records.enums.UserRoleEn;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.RoleRepository;
import cz.uhk.fim.redakcnisystem.usermanagement.repository.UserRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class AdminUserServiceImpl implements AdminUserService{

    private final UserRepository userRepository;
    private final RoleRepository roleRepository;

    @Override
    public ServiceResult<String> editUserRoles(Integer userId, List<UserRoleEn> updatedRoles) {
        Optional<User> userOptional = userRepository.findById(userId);
        if(userOptional.isEmpty())
            return Errors.userNotFound();
        List<UserRole> userRoles = userOptional.get().getRoles();
        List<UserRoleEn> currentUserRoles = new ArrayList<>();
        userRoles.forEach(userRole -> currentUserRoles.add(userRole.getRole()));
        currentUserRoles.forEach(currentUserRole -> {
            if(updatedRoles.contains(currentUserRole)){
                updatedRoles.remove(currentUserRole);
            } else {
                Optional<UserRole> userRoleToDelete = userRoles.stream().filter(userRole -> userRole.getRole() == currentUserRole).findFirst();
                userRoleToDelete.ifPresent(roleRepository::delete);
            }
        });
        if(!updatedRoles.isEmpty())
            updatedRoles.forEach(newRole -> {
                UserRole userRoleToAdd = new UserRole();
                userRoleToAdd.setUser(userOptional.get());
                userRoleToAdd.setRole(newRole);
                roleRepository.save(userRoleToAdd);
            });
        return ServiceResult.createServiceResult("Změna provedena", HttpStatus.OK);
    }
}
