import './App.css';
import React from 'react';
import 'bootstrap/dist/css/bootstrap.min.css';
import {BrowserRouter, Route, Routes} from 'react-router-dom';
import Navbar from './views/NavbarMenu';
import Login from './views/auth/Login';
import Registration from './views/auth/Registration';
import Articles from './views/articles/Articles';
import ShowProfile from "./views/user/ShowProfile";
import {ProtectedRoute} from "./components/ProtectedRoute";
import ShowArticle from "./views/articles/ShowArticle";
import EditUser from "./views/user/EditUser";
import NavbarAdmin from './views/admin/NavbarMenuAdmin';
import AdminHomePage from "./views/admin/AdminHomePage";
import ArticleEditor from "./views/admin/articlesAdmin/ArticleEditor";
import ArticleListAdmin from "./views/admin/articlesAdmin/ArticleListAdmin";
import UsersListAdmin from "./views/admin/usersAdmin/UsersListAdmin";
import GeneralSettingsAdmin from "./views/admin/settingsAdmin/GeneralSettingsAdmin";
import AuthorSettingsAdmin from "./views/admin/settingsAdmin/AuthorSettingsAdmin";
import {Settings} from "./components/SettingsManager";
import AboutAuthor from "./views/semistaticPages/AboutAuthor";
import Gallery from "./views/semistaticPages/Gallery";
import GallerySettingsAdmin from "./views/admin/settingsAdmin/GallerySettingsAdmin";
import Footer from "./views/Footer";
import StatisticsPage from "./views/admin/statistics/StatisticsPage";
import {Helmet, HelmetProvider} from "react-helmet-async";
import {BASE_URL} from "./components/AxiosConfig";
import UserStatistics from "./views/admin/statistics/UserStatistics";
import ArticleStatistics from "./views/admin/statistics/ArticleStatistics";

function App() {

    return (
        <HelmetProvider>
            <Helmet>
                <link rel={"icon"} href={BASE_URL+"/settings/favicon.ico"}/>
            </Helmet>
            <Settings>
            <BrowserRouter>
                {!window.location.pathname.match(/\/admin.*/g) ? <Navbar/> : <NavbarAdmin/>}
                <Routes>
                    {/* Admin routes */}
                    <Route path={'/admin/pridat-clanek'} element={
                        <ProtectedRoute childrenRoute={<ArticleEditor/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/upravit-clanek/:articleId'} element={
                        <ProtectedRoute childrenRoute={<ArticleEditor/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/clanky'} element={
                        <ProtectedRoute childrenRoute={<ArticleListAdmin/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/uzivatele'} element={
                        <ProtectedRoute childrenRoute={<UsersListAdmin/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/nastaveni/blog'} element={
                        <ProtectedRoute childrenRoute={<GeneralSettingsAdmin/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/nastaveni/tvurce'} element={
                        <ProtectedRoute childrenRoute={<AuthorSettingsAdmin/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/nastaveni/galerie'} element={
                        <ProtectedRoute childrenRoute={<GallerySettingsAdmin/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/statistiky'} element={
                        <ProtectedRoute childrenRoute={<StatisticsPage/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route exact path={'/admin/statistiky/uzivatel'} element={
                        <ProtectedRoute childrenRoute={<UserStatistics/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/statistiky/uzivatel/:userId'} element={
                        <ProtectedRoute childrenRoute={<UserStatistics/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route exact path={'/admin/statistiky/clanek'} element={
                        <ProtectedRoute childrenRoute={<ArticleStatistics/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>
                    <Route path={'/admin/statistiky/clanek/:articleId'} element={
                        <ProtectedRoute childrenRoute={<ArticleStatistics/>}
                                        mustBeLogged={true}
                                        alternativePath={"/"}
                                        roleList={["ADMIN"]}/>}/>

                    {/*Default routes*/}
                    <Route path='/' element={<Articles/>}/>

                    <Route path='/clanky' element={<Articles/>}/>
                    <Route path='/clanek/:articleId' element={<ShowArticle/>}/>
                    <Route path='/uzivatel/:userId' element={<ShowProfile/>}/>
                    <Route path='/autor' element={<AboutAuthor/>}/>
                    <Route path='/galerie' element={<Gallery/>}/>

                    {/*Pouze pro nepříhlášené */}
                    <Route path='/prihlaseni' element={
                        <ProtectedRoute childrenRoute={<Login/>} mustBeLogged={false} alternativePath={"/"}/>}/>
                    <Route path='/registrace' element={
                        <ProtectedRoute childrenRoute={<Registration/>} mustBeLogged={false} alternativePath={"/"}/>}/>

                    {/*Pouze pro příhlášené */}
                    <Route path='/upravit-profil' element={
                        <ProtectedRoute childrenRoute={<EditUser/>} mustBeLogged={true}
                                        alternativePath={"/prihlaseni"} roleList={["USER", "ADMIN"]}/>}/>
                    {/*Pouze pro admin role */}
                    <Route path={"/admin"} element={
                        <ProtectedRoute childrenRoute={<AdminHomePage/>} mustBeLogged={true}
                                        alternativePath={"/prihlaseni"} roleList={["ADMIN"]}/>
                    }/>
                </Routes>
                <Footer/>
            </BrowserRouter>
            </Settings>
        </HelmetProvider>
    );
}

export default App;
