import { Navigate } from 'react-router-dom';
import {useCookies} from "react-cookie";
import React from 'react';
import {getUserRoles, userRolesExists} from "./AuthManager";

/**
 * Slouží k tvorbě cest s oprávněným přístupem pouze pro přihlášené, případně pouze pro nepříhlašení.
 * @param childrenRoute Komponenta, která se předá v případě oprávněného přístupu
 * @param mustBeLogged Boolean, rozhoduje, zda se má kontrolovat přihlášení uživatele (true pouze pro přihlášené, false pouze pro nepříhlašené)
 * @param alternativePath String, slouží jako alternativní cesta pokud není splněna autorizace na základě požadavků
 * @param roleList List rolí ve stringu, dojde ke kontrole pouze v případě, že uživatel musí být přihlášen
 * */
export const ProtectedRoute = ({childrenRoute, mustBeLogged, alternativePath, roleList}) => {
    // získání id uživatele z cookies a ověření zda obsahuje validní hodnotu
    const [cookies] = useCookies('userId');
    let userLogged = Number.isInteger(Number(cookies.userId));
    let userRoles = userRolesExists() ? getUserRoles() : undefined;
    alternativePath = alternativePath ? alternativePath : "";
    // pokud uživatel musí být přihlášen ale není dojde k přesměrování na alternativní cestu
    if(mustBeLogged && !userLogged){
        return <Navigate to={alternativePath}/>
    }
    // pokud uživatel nesmí být přihlášen ale je dojde k přesměrování na alternativní cestu
    if(!mustBeLogged && userLogged){
        return <Navigate to={alternativePath}/>
    }
    // musí být přihlášen ale nemá požadovanou roli
    if(mustBeLogged && !roleList.some(role => userRoles.includes(role))){
        return <Navigate to={alternativePath}/>
    }
    // přesměrování na zabezpečenou komponentu
    return childrenRoute;
};