import React, { useEffect, useState} from "react";
import axios from "axios";
import {BASE_URL} from "./AxiosConfig";
import moment from "moment";

let defaultTitle = "Redakční systém";
let dateTimeFormat = "DD.MM.YYYY HH:mm:ss";
let dateFormat = "DD.MM.YYYY";
/**
 *
 * @param {String} newTitle new title to website
 * @param {boolean} withDefault should new title contain default title before newTitle
 */
export const setWebTitle = (newTitle, withDefault) => {
    document.title = withDefault ? (defaultTitle + " | " + newTitle) : newTitle;
}

export const formatDateTimeWithSettings = (dateTime) => {
    return moment(dateTime).format(dateTimeFormat);
}

export const formatDateWithSettings = (date) => {
    return moment(date).format(dateFormat);
}

export const SettingsContext = React.createContext({});

export const Settings = ({ children }) => {
    const [settings, setSettings] = useState({});
    useEffect(() => {
        loadSettings();
    }, []);

    function loadSettings(){
        axios.get(BASE_URL + "/settings/general").then(res => {
            setSettings(res.data.body);
        }).catch(err => {
            if(err.response.data.errors)
                console.log(err.response.data.errors);
        });
    }

    useEffect(() => {
        if(settings.blogName) {
            if(document.title.startsWith(defaultTitle)){
                document.title = document.title.replace(defaultTitle, settings.blogName);
            }
            defaultTitle = settings.blogName;
        }
        if(settings.dateTimeFormat){
            dateTimeFormat = settings.dateTimeFormat;
        }
    }, [settings]);

    return (<SettingsContext.Provider value={settings}>{children}</SettingsContext.Provider>);
}