import React, {useState, useEffect} from "react";
import CommentsFeed from "./newsFeedAdmin/CommentsFeed";
import {Button, Col, Container, Form, Row} from "react-bootstrap";
import RatingsFeed from "./newsFeedAdmin/RatingsFeed";
import moment from "moment";
import {setWebTitle} from "../../components/SettingsManager";

function AdminHomePage(){

    const [timePeriod, setTimePeriod] = useState({
        start: moment().subtract(1, "days").format("YYYY-MM-DDTHH:mm"),
        end: moment().add(1, "minutes").format("YYYY-MM-DDTHH:mm")
    });

    const [reloadComments, setReloadComments] = useState(true);
    const [reloadRatings, setReloadRatings] = useState(true);

    useEffect(() => {
        setWebTitle("Admin", true);
    }, []);

    function handleTimeChange(e){
        let {name, value} = e.target;
        setTimePeriod(prevState => ({
            ...prevState,
                [name]:value
        }));
    }

    function blurTimeCheckEndStart(){
        let tempPeriod = {...timePeriod};
        if(moment(tempPeriod.end).isBefore(moment(tempPeriod.start))) {
            let tempStart = tempPeriod.start;
            tempPeriod.start = tempPeriod.end;
            tempPeriod.end = tempStart;
            setTimePeriod(tempPeriod);
        }
    }

    function reload(){
        setReloadComments(true);
        setReloadRatings(true);
    }

    return(
        <div id={"content"}>
        <h1 className={"text-center"}>Přehled aktivity</h1>
            <Container>
                <Row>
                    <Container>
                        <Row className={"ms-auto me-auto w-50"}>
                            <Col className={"text-center"}>
                                <Form.Label>Začátek</Form.Label>
                                <Form.Control type={"datetime-local"}
                                              name={"start"}
                                              value={timePeriod.start}
                                              onChange={handleTimeChange}
                                              onBlur={blurTimeCheckEndStart}/>
                            </Col>
                            <Col className={"text-center"}>
                                <Form.Label>Konec</Form.Label>
                                <Form.Control type={"datetime-local"}
                                              name={"end"}
                                              value={timePeriod.end}
                                              onChange={handleTimeChange}
                                              onBlur={blurTimeCheckEndStart}/>
                            </Col>
                        </Row>
                        <Row className={"align-items-center justify-content-center"}>
                            <Button variant={"outline-success"}
                                    className={"w-25"}
                                    onClick={reload}>Načíst</Button>
                        </Row>
                    </Container>
                </Row>
                <Row>
                    <Col className={"me-0 ps-1 pe-1"}>
                        <br/>
                        <CommentsFeed timePeriod={timePeriod}
                                      reloadComments={reloadComments}
                                      setReloadComments={setReloadComments}/>
                    </Col>
                    <Col className={"ms-0 ps-1 pe-1"}>
                        <br/>
                        <RatingsFeed timePeriod={timePeriod}
                                     reloadRatings={reloadRatings}
                                     setReloadRatings={setReloadRatings}/>
                    </Col>
                </Row>
            </Container>
        </div>
    )
}

export default AdminHomePage;