import React, {useEffect, useState} from 'react';
import {Navbar, Container, Nav, NavDropdown} from 'react-bootstrap';
import axios from 'axios';
import {AxiosConfig, BASE_URL} from '../../components/AxiosConfig';
import {useCookies} from "react-cookie";
import {
    removeAuthToken, removeUserRoles,
    setAuthToken, setUserRoles,
    userCookiePathConf,
    userIdCookieName
} from "../../components/AuthManager";
import {Link} from "react-router-dom";
import SEOManager, {SEO_ROBOTS} from "../../components/SEOManager";

function NavbarMenuAdmin(){

    const [cookies, setCookie, removeCookie] = useCookies();
    const [user, setUser] = useState({
        data: {id: null},
        error: null
    });

    useEffect(()=> {
        axios.get(BASE_URL + "/auth/token", AxiosConfig()).then(res => {
            setAuthToken(res.data.body.token);
        }).catch(err => {
            removeAuthToken();
            removeUserRoles();
            removeCookie(userIdCookieName, userCookiePathConf);
            console.log(err);
        });
        axios.get(BASE_URL + '/user-management/current-user', AxiosConfig()).then(res => {
            setUser({
                data: res.data.body,
                error: res.data.errors,
            });
            if (res.data.body) {
                if (Number(res.data.body.id) !== cookies.userId) {
                    setCookie(userIdCookieName, res.data.body.id, userCookiePathConf);
                    setUserRoles(res.data.body.roles);
                }
            } else {
                removeCookie(userIdCookieName, userCookiePathConf);
                removeUserRoles();
            }
        }).catch(err => {
            console.log(err);
            window.location.href = "/prihlaseni";
        });
    }, [cookies[userIdCookieName], window.location.pathname]);
    

    return (
        <>
            <SEOManager description={"Stránka admina pro správu systému"}
                        robots={SEO_ROBOTS.NOINDEX_NOFOLLOW}/>
            <Navbar collapseOnSelect expand="lg" bg="dark" variant="dark">
                <Container>
                    <Navbar.Brand as={Link} to="/admin">Admin sekce</Navbar.Brand>
                    <Navbar.Toggle aria-controls="responsive-navbar-nav"/>
                    <Navbar.Collapse id="responsive-navbar-nav">
                        <Nav className="me-auto">
                            <Nav.Link as={Link} to="/admin/pridat-clanek">Přidat článek</Nav.Link>
                            <Nav.Link as={Link} to={"/admin/clanky"}>Články</Nav.Link>
                            <Nav.Link as={Link} to={"/admin/uzivatele"}>Uživatelé</Nav.Link>
                            <NavDropdown title={"Statistiky"}>
                                <NavDropdown.Item as={Link} to={"/admin/statistiky"}>Obecné statistiky</NavDropdown.Item>
                                <NavDropdown.Item as={Link} to={"/admin/statistiky/uzivatel"}>Statistika o uživateli</NavDropdown.Item>
                                <NavDropdown.Item as={Link} to={"/admin/statistiky/clanek"}>Statistika o článku</NavDropdown.Item>
                            </NavDropdown>
                            <NavDropdown title={"Nastavení"}>
                                <NavDropdown.Item as={Link} to={"/admin/nastaveni/blog"}>Blog</NavDropdown.Item>
                                <NavDropdown.Divider />
                                <NavDropdown.Item as={Link} to={"/admin/nastaveni/tvurce"}>O tvůrci</NavDropdown.Item>
                                <NavDropdown.Item as={Link} to={"/admin/nastaveni/galerie"}>Galerie</NavDropdown.Item>
                            </NavDropdown>
                        </Nav>
                            <Nav>
                                <Navbar.Text>Přihlášen jako: <b>{user.data.username}</b></Navbar.Text>
                                <Nav.Link href={"/"}>Uživatelská sekce</Nav.Link>
                            </Nav>
                    </Navbar.Collapse>
                </Container>
            </Navbar>
        </>
    )
}

export default NavbarMenuAdmin;