import React, {useState, useEffect} from 'react';
import {Button, Stack, Table, Form} from "react-bootstrap";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import CustomModal, {CustomModalTypes, CustomYesNoModal} from "../../viewComponents/CustomModal";
import {Link, useNavigate} from "react-router-dom";
import {formatDateTimeWithSettings, setWebTitle} from "../../../components/SettingsManager";
import CustomPagination from "../../viewComponents/CustomPagination";

function ArticleListAdmin() {

    const navigate = useNavigate();
    const [loadArticles, setLoadArticles] = useState(true);
    const [articles, setArticles] = useState([{
        id: "",
        title: "",
        visibility: null,
        createTime: "",
        lastUpdateTime: "",
        viewCount: undefined
    }]);
    const [perPage, setPerPage] = useState(10);
    const [currentPage, setCurrentPage] = useState(1);

    const [showYesNoModal, setShowYesNoModal] = useState(false);
    const [modalYesNoData, setModalYesNoData] = useState({
        header: "",
        text: "",
        buttonCancelType: null,
        buttonConfirmType: null,
        buttonCancelText: null,
        buttonConfirmText: null,
        articleId: null,
        operation: null
    });
    const [showModal, setShowModal] = useState(false);
    const [modalData, setModalData] = useState({
        header: "",
        text: "",
        type: null,
        buttonText: null
    });

    const ModalYesNoOperation = {
        Delete: "delete",
        ChangeVisibility: "visibility",
        Other: null
    }

    const [searchText, setSearchText] = useState("");

    useEffect(() => {
        setWebTitle("Admin články", true);
    }, []);

    useEffect(() => {
        if (loadArticles) {
            setLoadArticles(false);
            axios.get(BASE_URL + "/content/admin/article", AxiosConfig()).then(res => {
                setArticles(res.data.body.reverse());
            }).catch(err => {
                setModalData({
                    header: "Načtení článků",
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    type: CustomModalTypes.Danger
                });
                setShowModal(true);
            });
        }
    }, [loadArticles]);

    function sortTable(e) {
        const attributeName = e.currentTarget.getAttribute("name");
        let element = document.getElementById("icon-" + attributeName);
        let order = element.classList.contains("fa-caret-up") ? 1 : -1;
        //console.log("LOG: Sorting " + attributeName + " with order " + order + " " + (order === 1 ? "UP" : "DOWN"));
        element.classList.contains("fa-caret-up")
            ? element.classList.replace("fa-caret-up", "fa-caret-down")
            : element.classList.replace("fa-caret-down", "fa-caret-up");

        let tempData = [...articles];
        tempData = tempData.sort();
        tempData = tempData.sort((a, b) => {
            if (attributeName.toLowerCase().includes("time")) {
                return (
                    (order > 0)
                        ? (new Date(a[attributeName]) - new Date(b[attributeName]))
                        : (new Date(b[attributeName]) - new Date(a[attributeName]))
                );
            } else if (typeof a[attributeName] === "boolean") {
                return (
                    (order > 0)
                        ? a[attributeName] - b[attributeName]
                        : b[attributeName] - a[attributeName]
                );
            } else if (typeof a[attributeName] === "string") {
                return (
                    (order < 0)
                        ? a[attributeName].localeCompare(b[attributeName])
                        : b[attributeName].localeCompare(a[attributeName])
                );
            } else {
                return (
                    (order < 0)
                        ? a[attributeName] - b[attributeName]
                        : b[attributeName] - a[attributeName]
                )
            }
        });
        setArticles(tempData);
    }

    function confirmYesNoModal(articleId, operation) {
        switch (operation) {
            case ModalYesNoOperation.Delete:
                deleteArticle(articleId);
                break;
            case ModalYesNoOperation.ChangeVisibility:
                changeVisibility(articleId);
                break;
            case ModalYesNoOperation.Other:
                break;
        }
    }

    function changeVisibility(articleId) {
        let article = (articles.find(value => value.id === articleId));
        axios.patch(BASE_URL + "/content/article/" + articleId + "/visibility", JSON.stringify(!article.visibility), AxiosConfig()).then(res => {
            setShowYesNoModal(false);
            setModalData({
                header: "Změna viditelnosti.",
                text: "Změna viditelnosti u článku \"" + article.title + "\". " + res.data.body,
                type: CustomModalTypes.Success,
                buttonText: "Hotovo"
            });
            setShowModal(true);
            setLoadArticles(true);
        }).catch(err => {
            setShowYesNoModal(false);
            setModalData({
                header: "Změna viditelnosti.",
                text: "Změna viditelnosti u článku \"" + article.title + "\". " + err.response.data.errors.map(err => err.message).join(" "),
                type: CustomModalTypes.Danger,
                buttonText: "Zavřít"
            });
            setShowModal(true);
        });
    }

    function confirmChangeVisibility(articleId) {
        let article = articles.find(value => value.id === articleId);
        setModalYesNoData({
            header: "Změna viditelnosti",
            text: "Chcete změnit viditelnost u článku \"" + article.title +
                "\"? Aktuálně je článek " + (article.visibility ? "viditelný" : "skrytý") + ".",
            buttonCancelType: CustomModalTypes.Danger,
            buttonConfirmType: CustomModalTypes.Success,
            buttonCancelText: "Ne",
            buttonConfirmText: "Ano",
            articleId: articleId,
            operation: ModalYesNoOperation.ChangeVisibility
        });
        setShowYesNoModal(true);
    }

    function deleteArticle(articleId) {
        let article = articles.find(value => value.id === articleId);
        axios.delete(BASE_URL + "/content/article/" + articleId, AxiosConfig()).then(res => {
            setShowYesNoModal(false);
            setModalData({
                header: "Odebrání článku.",
                text: "Odebrání článku \"" + article.title + "\". " + res.data.body,
                type: CustomModalTypes.Success,
                buttonText: "Hotovo"
            });
            setShowModal(true);
            setLoadArticles(true);
        }).catch(err => {
            setShowYesNoModal(false);
            setModalData({
                header: "Odebrání článku.",
                text: "Odebrání článku \"" + article.title + "\". " + err.response.data.errors.map(err => err.message).join(" "),
                type: CustomModalTypes.Danger,
                buttonText: "Zavřít"
            });
            setShowModal(true);
        });
    }

    function confirmDeleteArticle(articleId) {
        let article = articles.find(value => value.id === articleId);
        setModalYesNoData({
            header: "Odebrání článku",
            text: "Chcete odebrat článek \"" + article.title +
                "\"? Odebrání článku není možné vrátit.",
            buttonCancelType: CustomModalTypes.Danger,
            buttonConfirmType: CustomModalTypes.Success,
            buttonCancelText: "Zrušit",
            buttonConfirmText: "Odebrat článek",
            articleId: articleId,
            operation: ModalYesNoOperation.Delete
        });
        setShowYesNoModal(true);
    }


    return (
        <div id={"content"}>
            <CustomModal modalData={modalData}
                         showModal={showModal}
                         onClose={() => setShowModal(false)}/>
            <CustomYesNoModal modalData={modalYesNoData}
                              showModal={showYesNoModal}
                              onCancel={() => setShowYesNoModal(false)}
                              onConfirm={() => confirmYesNoModal(modalYesNoData.articleId, modalYesNoData.operation)}/>
            <br/>
            <Stack direction={"horizontal"} gap={2} className={"admin-content-list"}>
                <Button className={"ml-auto"}
                        onClick={() => navigate("/admin/pridat-clanek")}
                        variant={"outline-success"}>Přidat článek <i className={"fa-solid fa-plus"}/></Button>
                <Button className={"ml-auto"}
                        onClick={() => {
                            setLoadArticles(true);
                            setSearchText("");
                        }}
                        variant={"outline-dark"}>Obnovit <i className={"fa-solid fa-rotate"}/></Button>
                <Form.Label>Vyhledat:</Form.Label>
                <Form.Control type={"text"}
                              className={"w-auto"}
                              placeholder={"Vyhledávání"}
                              onChange={(e) => setSearchText(e.target.value)}
                              value={searchText}/>
                <Form.Label className={"ms-auto"}>Zobrazení na stránce</Form.Label>
                <Form.Select className={"w-auto"} onChange={(e) => {
                    setPerPage(e.target.value);
                }}>
                    <option value={10}>10</option>
                    <option value={20}>20</option>
                    <option value={40}>40</option>
                    <option value={60}>60</option>
                </Form.Select>
            </Stack>

            <Table striped bordered responsive>
                <thead>
                <tr>
                    <th>#</th>
                    <th>ID</th>
                    <th name={"title"} onClick={sortTable}>Titulek
                        <i id={"icon-title"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"createTime"} onClick={sortTable}>Datum přidání
                        <i id={"icon-createTime"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"lastUpdateTime"} onClick={sortTable}>Datum úpravy
                        <i id={"icon-lastUpdateTime"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"viewCount"} onClick={sortTable}>
                        Počet zobrazení
                        <i id={"icon-viewCount"} style={{float: "right"}} className="fa-solid fa-caret-up"/>
                    </th>
                    <th name={"visibility"} onClick={sortTable}>Viditelný
                        <i id={"icon-visibility"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th>Statistiky</th>
                    <th>Upravit</th>
                    <th>Zobrazení</th>
                    <th>Odebrat</th>
                </tr>
                </thead>
                <tbody>
                {articles ? articles.filter(a => {return a.title.toLowerCase().includes(searchText.toLowerCase())})
                    .slice(((currentPage - 1) * perPage), (currentPage * perPage))
                    .map((article, index) => {
                    return (
                        <tr key={index}>
                            <td>{(index + 1) + ((currentPage - 1) * perPage)}</td>
                            <td>{article.id}</td>
                            <td><a href={"/clanek/" + article.id}>{article.title}</a></td>
                            <td>{formatDateTimeWithSettings(article.createTime)}</td>
                            <td>{formatDateTimeWithSettings(article.lastUpdateTime)}</td>
                            <td>{article.viewCount}</td>
                            <td>{article.visibility
                                ? <><span>Ano</span><i className="fa-solid fa-eye float-end"/></>
                                : <><span>Ne</span><i className="fa-solid fa-eye-slash float-end"/></>}
                            </td>
                            <td>
                                <Link to={"/admin/statistiky/clanek/"+article.id}>Zobrazit</Link>
                            </td>
                            <td>
                                <a href={"/admin/upravit-clanek/" + article.id}>Upravit</a>
                                <i className="fa-solid fa-pen-to-square float-end"/>
                            </td>
                            <td>
                                <a href={"#"} onClick={() => confirmChangeVisibility(article.id)}>
                                    {article.visibility
                                        ? <><span>Skrýt</span><i className="fa-solid fa-eye-slash float-end"/></>
                                        : <><span>Zobrazit</span><i className="fa-solid fa-eye float-end"/></>}
                                </a>
                            </td>
                            <td><a href={"#"} onClick={() => confirmDeleteArticle(article.id)}>Smazat</a></td>
                        </tr>
                    )
                }) : ""}
                </tbody>
                <tfoot>
                <tr>
                    <td colSpan={11} className={"text-center"}>
                        <CustomPagination maxItemsPerPage={perPage}
                                          defaultPage={1}
                                          numOfItems={articles.filter(a => {return a.title.toLowerCase().includes(searchText.toLowerCase())}).length}
                                          reloadDependency={articles}
                                          onChange={(e) => setCurrentPage(e)}/>
                    </td>
                </tr>
                </tfoot>
            </Table>
        </div>
    )
}

export default ArticleListAdmin;