import React, {useEffect, useState} from "react";
import moment from "moment";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import {Table} from "react-bootstrap";
import {formatDateTimeWithSettings} from "../../../components/SettingsManager";


function CommentsFeed({timePeriod, reloadComments, setReloadComments}) {

    const [commentsFeedData, setCommentsFeedData] = useState([{
        userId: null,
        username: "",
        commentId: null,
        comment: "",
        createTime: null,
        articleId: null,
        title: "",
    }]);

    useEffect(() => {
        setReloadComments(false);
    });

    if (reloadComments) {
        let tempConfig = {...AxiosConfig()};
        tempConfig.params = {
            timeStart: moment(timePeriod.start).toISOString(),
            timeEnd: moment(timePeriod.end).toISOString()
        };
        axios.get(BASE_URL + "/admin/content/feed/comment", tempConfig).then(res => {
            setCommentsFeedData(res.data.body);
        }).catch(err => {
            if(err.response.data.errors)
                console.log(err.response.data.errors);
        });
    }

    return (
        <div className={"feed-scroll"}>
        <Table responsive striped bordered size={"sm"}>
            <thead>
            <tr align={"center"}>
                <th>#</th>
                <th>Uživatel</th>
                <th>Článek</th>
                <th>Komentář</th>
                <th>Čas</th>
            </tr>
            </thead>
            <tbody>
            {commentsFeedData ? commentsFeedData.map((comment, index) => {
                    return (
                        <tr key={index}>
                            <td>{index + 1}</td>
                            <td><a href={"/uzivatel/" + comment.userId}>{comment.username}</a></td>
                            <td><a href={"/clanek/" + comment.articleId}>{comment.title}</a></td>
                            <td onClick={event => {
                                if (comment.comment.length > 100) {
                                    if (event.currentTarget.innerHTML.length === comment.comment.length) {
                                        event.currentTarget.innerHTML = comment.comment.substring(0, 100) + "<br/>... Zobrazit vše";
                                    } else {
                                        event.currentTarget.innerHTML = comment.comment;
                                    }
                                }
                            }}>
                                {comment.comment.length > 100
                                ? (comment.comment.substring(0,100) +"... Zobrazit vše")
                                : comment.comment}
                            </td>
                            <td>{formatDateTimeWithSettings(comment.createTime)}</td>
                        </tr>
                    )
                }) :
                <tr>
                    <td colSpan={5} className={"text-center"}>Žádný obsah za zvolené obodbí.</td>
                </tr>
            }
            </tbody>
        </Table>
        </div>
    )
}

export default CommentsFeed;