import React, {useEffect, useState} from "react";
import {Table} from "react-bootstrap";
import moment from "moment";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import axios from "axios";
import StarRatings from "react-star-ratings/build/star-ratings";
import {formatDateTimeWithSettings} from "../../../components/SettingsManager";

function RatingsFeed({timePeriod, reloadRatings, setReloadRatings}) {

    const [ratingFeedData, setRatingFeedData] = useState([{
        userId: null,
        username: "",
        articleId: null,
        title: "",
        ratingUser: null,
        createTime: ""
    }]);

    useEffect(() => {
        setReloadRatings(false);
    });

    if(reloadRatings){
        let tempConfig = {...AxiosConfig()};
        tempConfig.params = {
            timeStart: moment(timePeriod.start).toISOString(),
            timeEnd: moment(timePeriod.end).toISOString()
        };
        axios.get(BASE_URL + "/admin/content/feed/rating", tempConfig).then(res => {
            setRatingFeedData(res.data.body);
        }).catch(err => {
            if(err.response.data.errors)
                console.log(err.response.data.errors);
        });
    }

    return (
        <div className={"feed-scroll"}>
        <Table responsive striped bordered size={"sm"}>
            <thead>
            <tr align={"center"}>
                <th>#</th>
                <th>Uživatel</th>
                <th>Článek</th>
                <th>Hodnocení</th>
                <th>Čas</th>
            </tr>
            </thead>
            <tbody>
            {ratingFeedData ? ratingFeedData.map((rating, index) => {
                    return (
                        <tr key={index}>
                            <td>{index + 1}</td>
                            <td><a href={"/uzivatel/" + rating.userId}>{rating.username}</a></td>
                            <td><a href={"/clanek/" + rating.articleId}>{rating.title}</a></td>
                            <td valign={"middle"} align={"center"}>
                                <StarRatings rating={Number(rating.ratingUser)}
                                             starRatedColor={"rgb(18,179,0)"}
                                             numberOfStars={5}
                                             starDimension={"15px"}
                                             starSpacing={"0px"}/>
                            </td>
                            <td>{formatDateTimeWithSettings(rating.createTime)}</td>
                        </tr>
                    )
                })
                :
                <tr>
                    <td colSpan={5} className={"text-center"}>Žádná hodnocení za dané období.</td>
                </tr>
            }
            </tbody>
        </Table>
        </div>
    );
}

export default RatingsFeed;