import React, {useEffect, useState} from "react";
import {Button, Col, Form, Row} from "react-bootstrap";
import ReactQuill from "react-quill";
import {EditorFormats, EditorModules} from "../articlesAdmin/EditorSettings";
import parse from "html-react-parser";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import CustomModal, {CustomModalTypes} from "../../viewComponents/CustomModal";


function AuthorSettingsAdmin() {

    const [autoRefreshPreview, setAutoRefreshPreview] = useState(false);
    const [authorPageData, setAuthorPageData] = useState({
        text: "",
        facebookUrl: "",
        instagramUrl: "",
        twitterUrl: "",
        linkedInUrl: "",
        socialNetworkHeading: "Mé sociální sítě"
    });
    const [showModal, setShowModal] = useState(false);
    const [modalData, setModalData] = useState({
        header:"",
        text:"",
        type:CustomModalTypes.Info
    });

    useEffect(() => {
        axios.get(BASE_URL + "/settings/author-page", AxiosConfig()).then(res => {
            setAuthorPageData(res.data.body);
        }).catch(onError);
    }, []);

    function handleDataChange(e) {
        let {value, name} = e.target;
        if (value === "" || value === undefined)
            value = null;
        setAuthorPageData(prevState => ({
            ...prevState,
            [name]: value
        }));
    }

    function submitPage(){
        axios.post(BASE_URL+ "/settings/author-page", authorPageData, AxiosConfig()).then(res => {
            setModalData({
                text: res.data.body,
                header: "Úspěch",
                type: CustomModalTypes.Success
            });
            setShowModal(true);
        }).catch(onError);
    }

    function onError(err){
        if(err.response.data.errors) {
            setModalData({
                text: err.response.data.body,
                header: "Chyba",
                type: CustomModalTypes.Danger
            });
            setShowModal(true);
        }
    }

    return (
        <div id={"content"}>
            <CustomModal modalData={modalData} showModal={showModal} onClose={()=>setShowModal(false)}/>
            <h1>Nastavení o tvůrci</h1>
            <Form>
                <h3>Sociální sítě</h3>
                <Row>
                    <Form.Group as={Col}>
                        <Form.Label>Nadpis k sociálním sítím:</Form.Label>
                        <Form.Control type={"text"} name={"socialNetworkHeading"}
                                      value={authorPageData.socialNetworkHeading} onChange={handleDataChange}/>
                    </Form.Group>
                </Row>
                <Row>
                <Form.Group as={Col}>
                    <Form.Label>Facebook</Form.Label>
                    <Form.Control type={"text"} name={"facebookUrl"} value={authorPageData.facebookUrl} onChange={handleDataChange}/>
                </Form.Group>
                <Form.Group as={Col}>
                    <Form.Label>Instagram</Form.Label>
                    <Form.Control type={"text"} name={"instagramUrl"} value={authorPageData.instagramUrl} onChange={handleDataChange}/>
                </Form.Group>
                </Row>
                <Row>
                    <Form.Group as={Col}>
                        <Form.Label>X / Twitter</Form.Label>
                        <Form.Control type={"text"} name={"twitterUrl"} value={authorPageData.twitterUrl} onChange={handleDataChange}/>
                    </Form.Group>
                    <Form.Group as={Col}>
                        <Form.Label>LinkedIn</Form.Label>
                        <Form.Control type={"text"} name={"linkedInUrl"} value={authorPageData.linkedInUrl} onChange={handleDataChange}/>
                    </Form.Group>
                </Row>
                <Form.Group>
                    <Form.Label as={"h3"}>Hlavní obsah stránky:</Form.Label>
                    <ReactQuill className={"editor-admin"}
                                value={authorPageData.text}
                                theme={"snow"}
                                modules={EditorModules}
                                formats={EditorFormats}
                                id={"test"}
                                onChange={(content) => {
                                    setAuthorPageData(prevState => ({
                                        ...prevState,
                                        text: content
                                    }));
                                }}>
                    </ReactQuill>
                </Form.Group>
                <Row className={"justify-content-center"}>
                    <Button variant={"outline-success"}
                            className={"w-25"}
                            onClick={submitPage}>Uložit</Button>
                </Row>
            </Form>
            <div>
                <h1>Ukázka</h1>
                <Form.Switch label={"Zobrazení ukázky"}
                             checked={autoRefreshPreview}
                             onChange={() => setAutoRefreshPreview(!autoRefreshPreview)}/>
                {autoRefreshPreview ?
                    <div className={"border"}>
                        <div className={"ql-editor"}>{parse(authorPageData.text)}</div>
                        <div>
                            <h5>{authorPageData.socialNetworkHeading}</h5>
                            {authorPageData.facebookUrl
                                ? <><a href={authorPageData.facebookUrl} target={"_blank"}>
                                    <i className={"fa-brands fa-facebook"}/> Facebook</a> <br/></>
                                : ""}
                            {authorPageData.instagramUrl
                                ? <><a href={authorPageData.instagramUrl} target={"_blank"}>
                                    <i className={"fa-brands fa-instagram"}/> Instagram</a><br/></>
                                : ""}
                            {authorPageData.twitterUrl
                                ? <><a href={authorPageData.twitterUrl} target={"_blank"}>
                                    <i className={"fa-brands fa-twitter"}/> X / Twitter</a><br/></>
                                : ""}
                            {authorPageData.linkedInUrl
                                ? <><a href={authorPageData.linkedInUrl} target={"_blank"}>
                                    <i className={"fa-brands fa-linkedin"}/> LinkedIn</a><br/></>
                                : ""}
                        </div>
                    </div> : ""}
            </div>
        </div>
    )
}

export default AuthorSettingsAdmin;