import React, {useState, useEffect} from "react";
import {Button, Form, Table} from "react-bootstrap";
import axios from "axios";
import {AxiosConfig, AxiosConfigImage, BASE_URL} from "../../../components/AxiosConfig";
import CustomModal, {CustomModalTypes} from "../../viewComponents/CustomModal";
import moment from "moment";
import {useNavigate} from "react-router-dom";

function GeneralSettingsAdmin() {

    const [settingsData, setSettingsData] = useState({
        blogName: "Redakční systém",
        displayNumberOfRatings: true,
        displayNumberOfViewsArticle: true,
        displayUpdateCreateTime: "createAndUpdate",
        dateTimeFormat: "DD.MM.YYYY HH:mm:ss",
        enableAuthorPage: true,
        authorPageLinkText: "Autor",
        enableGalleryPage: false,
        galleryPageLinkText: "Galerie",
        footerFirstRow: "Děkuji za návštěvu.",
        footerSecondRow: "© " + new Date().getFullYear() + " Copyright: ",
        footerBackgroundColor: "#464646",
        footerTextColor: "#FAFAFA"
    });
    const [displayModal, setDisplayModal] = useState(false);
    const [modalData, setModalData] = useState({
        header: "",
        text: "",
        type: CustomModalTypes.Info
    });
    const [displayDateHelp, setDisplayDateHelp] = useState(false);
    const navigate = useNavigate();

    const [iconData, setIconData] = useState({
        data: undefined,
        url: undefined
    });

    useEffect(() => {
        axios.get(BASE_URL + "/settings/general", AxiosConfig()).then(res => {
            setSettingsData(res.data.body);
        }).catch(err => {
            if (err.response.data.errors) {
                setModalData({
                    header: "Chyba",
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    type: CustomModalTypes.Danger
                });
                setDisplayModal(true);
            }
        });
    }, []);

    function submitSettings() {
        axios.post(BASE_URL + "/settings/general", settingsData, AxiosConfig()).then(res => {
            setModalData({
                header: "Úspěch",
                text: res.data.body,
                type: CustomModalTypes.Success
            });
            setDisplayModal(true);
        }).catch(err => {
            setModalData({
                header: "Došlo k chybě",
                text: err.response.data.errors,
                type: CustomModalTypes.Danger
            });
            setDisplayModal(true);
        });
    }

    function handleFormChange(e) {
        let {name, value} = e.target;
        if (e.target.type === "checkbox") {
            value = e.target.checked;
        }
        setSettingsData(prevState => ({
            ...prevState,
            [name]: value
        }));
    }

    function uploadIcon() {
        if(iconData.data) {
            let tempConfig = {...AxiosConfigImage()};
            tempConfig.headers["Content-Type"] = "image/x-icon";
            axios.post(BASE_URL + "/settings/favicon.ico", iconData.data, tempConfig).then(res => {
                setModalData({
                    header: "Úspěch",
                    text: res.data.body,
                    type: CustomModalTypes.Success
                });
                setDisplayModal(true);
            }).catch(err => {
                setModalData({
                    header: "Chyba",
                    text: err.response.data.errors
                        ? err.response.data.errors.map(e => e.message).join(" ")
                        : "Došlo k neznámé chybě.",
                    type: CustomModalTypes.Danger
                });
                setDisplayModal(true);
            });
        }
    }

    function changeIconHandle(e) {
        let val = e.target.files[0];
        setIconData({
            url: URL.createObjectURL(val),
            data: val
        });
    }

    return (
        <div id={"content-short"}>
            <CustomModal onClose={() => {
                setDisplayModal(false);
                navigate(0);
            }}
                         showModal={displayModal}
                         modalData={modalData}/>
            <br/>
            <Table borderless={false} responsive className={"article-editor-survey-table"}>
                <thead>
                <tr>
                    <th className={"text-center h3"} colSpan={10}>Obecné nastavení</th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td colSpan={2}>Název blogu:</td>
                    <td colSpan={2}>
                        <Form.Control value={settingsData.blogName}
                                      name={"blogName"}
                                      onChange={handleFormChange}
                                      type={"text"}/>
                    </td>
                </tr>
                <tr>
                    <td>Ikona <img src={BASE_URL + "/settings/favicon.ico"} alt={"Aktuální ikona"}/></td>
                    <td>Náhled:
                        {iconData.url ? <img src={iconData.url} alt={"Náhled nové ikony"}/> : ""}
                    </td>
                    <td><Form.Control type={"file"} accept={"image/x-icon"} id={"icon-upload"}
                                      onChange={changeIconHandle}/></td>
                    <td>
                        <Button variant={"outline-success"}
                                onClick={uploadIcon}
                                disabled={!iconData.data}>
                            Uložit ikonu
                        </Button>
                    </td>
                </tr>
                </tbody>
                <thead>
                <tr>
                    <th className={"text-center h4"} colSpan={10}>Nastavení článků</th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td>Zobrazit počet hodnocení článku:</td>
                    <td>
                        <Form.Switch
                            checked={settingsData.displayNumberOfRatings}
                            name={"displayNumberOfRatings"}
                            onChange={handleFormChange}/>
                    </td>
                    <td>Zobrazit počet zobrazení článku:</td>
                    <td>
                        <Form.Switch
                            checked={settingsData.displayNumberOfViewsArticle}
                            name={"displayNumberOfViewsArticle"}
                            onChange={handleFormChange}/>
                    </td>
                </tr>
                </tbody>
                <thead>
                <tr>
                    <th className={"text-center h4"} colSpan={10}>Nastavení času</th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td colSpan={2}>Zobrazit čas u článku:</td>
                    <td colSpan={2}>
                        <Form.Select value={settingsData.displayUpdateCreateTime}
                                     name={"displayUpdateCreateTime"}
                                     onChange={handleFormChange}>
                            <option value={"createAndUpdate"}>Vytvoření a úprava</option>
                            <option value={"create"}>Vytvoření</option>
                            <option value={"update"}>Úprava</option>
                        </Form.Select>
                    </td>
                </tr>
                <tr>
                    <td>Formát času a data:</td>
                    <td>Náhled: {moment().format(settingsData.dateTimeFormat)}</td>
                    <td colSpan={2}>
                        <Form.Control type={"text"}
                                      value={settingsData.dateTimeFormat}
                                      name={"dateTimeFormat"}
                                      onFocus={() => setDisplayDateHelp(true)}
                                      onBlur={() => setDisplayDateHelp(false)}
                                      onChange={handleFormChange}/>
                        <Form.Text hidden={!displayDateHelp}>
                            Dostupné možnosti formátu, není nutné využít všechny.<br/>
                            Rok: YYYY (2022), YY (22) <br/>
                            Měsíc: M (1,12), MM (01,12) <br/>
                            Den: D (1,30), DD (01,30) <br/>
                            Hodina: H (8,12), HH (08,12)<br/>
                            Minuta: m (5,40), mm (05,40)<br/>
                            Vteřina: s (1,20), ss (01,20)<br/>
                            Oddělovače: , . : / -
                        </Form.Text>
                    </td>
                </tr>
                </tbody>
                <thead>
                <tr>
                    <th className={"text-center h4"} colSpan={10}>Patička</th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td>První řádek:</td>
                    <td colSpan={3}>
                        <Form.Control type={"text"}
                                      name={"footerFirstRow"}
                                      value={settingsData.footerFirstRow}
                                      onChange={handleFormChange}/>
                    </td>
                </tr>
                <tr>
                    <td>Druhý řádek:</td>
                    <td colSpan={3}>
                        <Form.Control type={"text"}
                                      name={"footerSecondRow"}
                                      value={settingsData.footerSecondRow}
                                      onChange={handleFormChange}/>
                    </td>
                </tr>
                <tr>
                    <td>Barva pozadí:</td>
                    <td>
                        <Form.Control type={"color"}
                                      title={"Vyber barvu pozadí."}
                                      name={"footerBackgroundColor"}
                                      value={settingsData.footerBackgroundColor}
                                      onChange={handleFormChange}/>
                    </td>
                    <td>Barva textu:</td>
                    <td>
                        <Form.Control type={"color"}
                                      title={"Vyber barvu textu."}
                                      name={"footerTextColor"}
                                      value={settingsData.footerTextColor}
                                      onChange={handleFormChange}/>
                    </td>
                </tr>
                </tbody>
                <thead>
                <tr>
                    <th className={"text-center h4"} colSpan={10}>Stránky</th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td>Stránka autora:</td>
                    <td><Form.Switch checked={settingsData.enableAuthorPage} name={"enableAuthorPage"}
                                     label={"Přístupná veřejně"} onChange={handleFormChange}/></td>
                    <td className={"text-end"}>Text v nabídce odkazující na stránku:</td>
                    <td><Form.Control type={"text"} value={settingsData.authorPageLinkText} name={"authorPageLinkText"}
                                      placeholder={"Autor"} onChange={handleFormChange}/></td>
                </tr>
                <tr>
                    <td>Galerie</td>
                    <td><Form.Switch checked={settingsData.enableGalleryPage} name={"enableGalleryPage"}
                                     label={"Přístupná veřejně"} onChange={handleFormChange}/></td>
                    <td className={"text-end"}>Text v nabídce odkazující na stránku:</td>
                    <td><Form.Control type={"text"} value={settingsData.galleryPageLinkText}
                                      name={"galleryPageLinkText"}
                                      placeholder={"Galerie"} onChange={handleFormChange}/></td>
                </tr>
                </tbody>
                <tfoot>
                <tr>
                    <td colSpan={10} className={"text-center"}>
                        <Button className={"w-25"}
                                variant={"outline-success"}
                                onClick={submitSettings}>Uložit</Button>
                    </td>
                </tr>
                </tfoot>
            </Table>
        </div>
    )
}

export default GeneralSettingsAdmin;