import React, {useState} from "react";
import {useParams} from "react-router";
import {Button, Form, Row, Table} from "react-bootstrap";
import {Link} from "react-router-dom";
import CustomDateRangeSelect from "../../viewComponents/CustomDateRangeSelect";
import SpecificArticleStatistics from "./SpecificArticleStatistics";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import axios from "axios";

function ArticleStatistics() {

    const [articleId, setArticleId] = useState(useParams()["articleId"]);
    const [dateRange, setDateRange] = useState({end: undefined, start: undefined});
    const [searchText, setSearchText] = useState("");
    const [searchedArticles, setSearchedArticles] = useState([]);

    function search(e){
        e.preventDefault();
        let tempConf ={...AxiosConfig()};
        tempConf.params = {search: searchText};
        axios.get(BASE_URL + "/admin/statistics/article", tempConf).then(res => {
            setSearchedArticles(res.data.body);
        }).catch(err => {
            console.log(err);
        })
    }

    return (
        <div id={"content"}>
            {articleId
                ? <>
                <br/>
                    <Row className={"justify-content-end"}>
                        <Button variant={"warning"}
                                className={"w-auto"}
                                as={Link}
                                to={"/admin/statistiky/clanek"}
                                onClick={() => setArticleId(undefined)}>
                            Zpět na výběr článků
                        </Button>
                    </Row>
                    <CustomDateRangeSelect onConfirmRange={(range) => setDateRange(range)}/>
                    <SpecificArticleStatistics dateRange={dateRange} articleId={articleId}/>
                </>
                : <>
                    <br/>
                    <Form onSubmit={search}>
                        <Row className={"flex-row align-items-center justify-content-center"}>
                        <Form.Label className={"w-auto"}>Vyhledat článek: </Form.Label>
                        <Form.Control className={"w-50"}
                                      type={"text"}
                                      value={searchText}
                                      onChange={(e) => setSearchText(e.target.value)}/>
                        <Button className={"w-auto"}
                                variant={"outline-success"}
                                type={"submit"}>
                            Vyhledat
                        </Button>
                        </Row>
                    </Form>
                    <br/>
                    <Table responsive>
                        <tbody>
                        {Array.isArray(searchedArticles) && searchedArticles.length > 0
                            ? searchedArticles.map((article, index) => {
                                return (
                                    <tr key={index}>
                                        <td>{Number(index+1)}</td>
                                        <td>{article.title}</td>
                                        <td>
                                            <Link to={"/admin/statistiky/clanek/"+article.id}
                                                  onClick={() => setArticleId(article.id)}>
                                                Zobrazit statistiky článku
                                            </Link>
                                        </td>
                                    </tr>
                                )
                            })
                            : <tr>
                                <td colSpan={4} className={"text-center"}>Nebylo nic nalezeno</td>
                            </tr>
                        }
                        </tbody>
                    </Table>
                </>
            }
        </div>
    )
}

export default ArticleStatistics;