import React, {useState, useEffect} from "react";
import {Chart} from "react-chartjs-2";
import {getDatesBetweenFromRange} from "../../../components/DateHelper";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import moment from "moment";


function ArticlesChart({dateRange}) {

    const [articleViewsDataset, setArticleViewDatasets] = useState({
        label: "Počet nových zobrazení všech článků",
        data: [],
        borderWidth: 1,
        borderColor: "#0000AA",
        backgroundColor: "#0000FF"
    });
    const [articleCommentsDataset, setArticleCommentsDataset] = useState({
        label: "Počet nových komentářů všech článků",
        data: [],
        borderWidth: 1,
        borderColor: "#00AAAA",
        backgroundColor: "#00FFFF"
    });
    const [articleRatingsDataset, setArticleRatingsDataset] = useState({
        label: "Počet nových hodnocení všech článků",
        data: [],
        borderWidth: 1,
        borderColor: "#006666",
        backgroundColor: "#008888"
    });
    const [labels, setLabels] = useState(getDatesBetweenFromRange(dateRange));

    useEffect(() => {
        let tempConfig = {...AxiosConfig()};
        tempConfig.params = {
            startDate: moment(dateRange.start).toISOString(),
            endDate: moment(dateRange.end).toISOString()
        };
        axios.get(BASE_URL + "/admin/statistics/article/comments", tempConfig).then(res => {
            let newState = {...articleCommentsDataset};
            newState.data = res.data.body.map(o => o.count);
            setLabels(res.data.body.map(o => moment(o.date).format("DD.MM.YYYY")));
            setArticleCommentsDataset(newState);
        }).catch(err => {
            console.log(err);
        });
        axios.get(BASE_URL + "/admin/statistics/article/ratings", tempConfig).then(res => {
            let newState = {...articleRatingsDataset};
            newState.data = res.data.body.map(o => o.count);
            setLabels(res.data.body.map(o => moment(o.date).format("DD.MM.YYYY")));
            setArticleRatingsDataset(newState);
        }).catch(err => {
            console.log(err);
        });
        axios.get(BASE_URL + "/admin/statistics/article/views", tempConfig).then(res => {
            let newState = {...articleViewsDataset};
            newState.data = res.data.body.map(o => o.count);
            setLabels(res.data.body.map(o => moment(o.date).format("DD.MM.YYYY")));
            setArticleViewDatasets(newState);
        }).catch(err => {
            console.log(err);
        });
    }, [dateRange]);

    return (
        <Chart type={"line"}
               data={{
                   labels: labels,
                   datasets: [articleViewsDataset, articleCommentsDataset, articleRatingsDataset]
               }}/>
    )
}

export default ArticlesChart;