import React, {useEffect, useState} from "react";
import {Col, Container, Row, Spinner, Table} from "react-bootstrap";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import moment from "moment";
import axios from "axios";
import {getDatesBetweenFromRange} from "../../../components/DateHelper";
import {Chart} from "react-chartjs-2";
import StarRatings from "react-star-ratings/build/star-ratings";
import {Link} from "react-router-dom";


function SpecificArticleStatistics({dateRange, articleId}) {

    const [labels, setLabels] = useState(getDatesBetweenFromRange(dateRange));
    const [otherArticleInfo, setOtherArticleInfo] = useState({
        id: undefined,
        title: "",
        createTime: undefined,
        lastUpdateTime: undefined,
        rating: undefined,
        countOfSaved: undefined,
        countOfAllComments: undefined,
        countOfAllRatings: undefined,
    });
    const [sumOfEachRatingValue, setSumOfEachRatingValue] = useState([]);
    const [newArticleRatingsDataset, setNewArticleRatingsDataset] = useState({
        label: "Hodnocení",
        data: [],
        borderWidth: 1,
        borderColor: "#00CC00",
        backgroundColor: "#00DD00"
    });
    const [allArticleRatingsDataset, setAllArticleRatingsDataset] = useState({
        label: "Hodnocení",
        data: [],
        borderWidth: 1,
        borderColor: "#00AA00",
        backgroundColor: "#009900"
    });
    const [newArticleCommentsDataset, setNewArticleCommentsDataset] = useState({
        label: "Komentáře",
        data: [],
        borderWidth: 1,
        borderColor: "#0000CC",
        backgroundColor: "#0000DD"
    });
    const [allArticleCommentsDataset, setAllArticleCommentsDataset] = useState({
        label: "Komentáře",
        data: [],
        borderWidth: 1,
        borderColor: "#0000AA",
        backgroundColor: "#000099"
    });
    const [newArticleViewsDataset, setNewArticleViewsDataset] = useState({
        label: "Zobrazení",
        data: [],
        borderWidth: 1,
        borderColor: "#CC0000",
        backgroundColor: "#DD0000"
    });
    const [allArticleViewsDataset, setAllArticleViewsDataset] = useState({
        label: "Zobrazení",
        data: [],
        borderWidth: 1,
        borderColor: "#AA0000",
        backgroundColor: "#990000"
    });
    const [loading, setLoading] = useState(true);
    const [isSurveyExists, setSurveyExists] = useState(false);
    const [newSurveyVotesDataset, setNewSurveyVotesDataset] = useState({
        label: "Hlasy v anketě",
        data: [],
        borderWidth: 1,
        borderColor: "#880088",
        backgroundColor: "#660066"
    });
    const [surveyOptionsWithVotes, setSurveyOptionsWithVotes] = useState([{
        text: undefined,
        position: undefined,
        count: undefined
    }]);
    const [surveyText, setSurveyText] = useState(undefined);
    const [surveyCountAllVotes, setSurveyCountAllVotes] = useState(undefined);

    useEffect(() => {
        if (articleId && dateRange) {
            setLoading(true);
            let tempConfig = {...AxiosConfig()};
            tempConfig.params = {
                startDate: moment(dateRange.start).toISOString(),
                endDate: moment(dateRange.end).toISOString()
            };
            axios.get(BASE_URL + "/admin/statistics/article/" + articleId, tempConfig).then(res => {
                if(res.data.body.survey){
                    setNewSurveyVotesDataset(prevState => ({
                        ...prevState,
                        data: res.data.body.survey.votesPerDay.map(a => a.count)
                    }));
                    setSurveyCountAllVotes(res.data.body.survey.countOfAllVotes);
                    setSurveyText(res.data.body.survey.surveyText);
                    setSurveyOptionsWithVotes(res.data.body.survey.surveyOptionsWithVotes);
                    setSurveyExists(true);
                } else {
                    setSurveyExists(false);
                }
                setSumOfEachRatingValue(res.data.body.sumOfEachRatingValue);
                setOtherArticleInfo({
                    id: res.data.body.id,
                    title: res.data.body.title,
                    rating: res.data.body.rating,
                    createTime: res.data.body.createTime,
                    lastUpdateTime: res.data.body.lastUpdateTime,
                    countOfSaved: res.data.body.countOfSaved,
                    countOfAllComments: res.data.body.countOfAllComments,
                    countOfAllRatings: res.data.body.countOfAllRatings
                });

                setNewArticleRatingsDataset(prevState => ({
                    ...prevState,
                    data: res.data.body.newArticleRatings.map(a => a.count)
                }));
                setAllArticleRatingsDataset(prevState => ({
                    ...prevState,
                    data: res.data.body.allArticleRatings.map(a => a.count)
                }));

                setNewArticleCommentsDataset(prevState => ({
                    ...prevState,
                    data: res.data.body.newArticleComments.map(a => a.count)
                }));
                setAllArticleCommentsDataset(prevState => ({
                    ...prevState,
                    data: res.data.body.allArticleComments.map(a => a.count)
                }));

                setNewArticleViewsDataset(prevState => ({
                    ...prevState,
                    data: res.data.body.newArticleViews.map(a => a.count)
                }));
                setAllArticleViewsDataset(prevState => ({
                    ...prevState,
                    data: res.data.body.allArticleViews.map(a => a.count)
                }));
                setLabels(res.data.body.allArticleViews.map(a => moment(a.date).format("DD.MM.YYYY")));
                setLoading(false);
            }).catch(err => {
                console.log(err);
            });
        }
    }, [dateRange, articleId]);

    if (loading)
        return (
            <Row className={"justify-content-center"}>
                <br/>
                <Spinner />
                <h3 className={"text-center"}>Probíhá načítání</h3>
            </Row>
        )
    if (!loading)
        return (
            <Container>
                <Row>
                    <h3 className={"text-center"}>Informace o článku: {otherArticleInfo.title}</h3>
                </Row>
                <Row className={"admin-stats-row"}>
                    <Col>
                        <Chart type={"line"} data={{
                            labels: labels,
                            datasets: [
                                allArticleRatingsDataset,
                                allArticleCommentsDataset,
                                allArticleViewsDataset
                            ]
                        }} options={{
                            plugins: {
                                title: {
                                    display: true,
                                    text: "Údaje celkem"
                                }
                            }
                        }}/>
                    </Col>
                    <Col>
                        <Chart type={"line"} data={{
                            labels: labels,
                            datasets: [
                                newArticleRatingsDataset,
                                newArticleCommentsDataset,
                                newArticleViewsDataset,
                                isSurveyExists ? newSurveyVotesDataset : ""
                            ]
                        }} options={{
                            plugins: {
                                title: {
                                    display: true,
                                    text: "Nové údaje"
                                }
                            }
                        }}/>
                    </Col>
                </Row>
                <Row className={"admin-stats-row"}>
                    <Col>
                        <Chart type={"bar"} data={{
                            labels: [1, 2, 3, 4, 5],
                            datasets: [
                                {
                                    label: "Počet hodnocení podle hodnoty",
                                    backgroundColor: "#009999",
                                    data: sumOfEachRatingValue
                                }
                            ]
                        }}/>
                    </Col>
                    <Col>
                        <Table responsive>
                            <thead>
                            <tr>
                                <th className={"text-center"} colSpan={2}>Ostatní údaje</th>
                            </tr>
                            </thead>
                            <tbody>
                            <tr>
                                <td>Název článku: </td>
                                <td>{otherArticleInfo.title}</td>
                            </tr>
                            <tr>
                                <td>Datum a čas přidání článku:</td>
                                <td>{moment(otherArticleInfo.createTime).format("HH:mm DD.MM.YYYY")}</td>
                            </tr>
                            <tr>
                                <td>Datum a čas poslední úpravy:</td>
                                <td>{moment(otherArticleInfo.lastUpdateTime).format("HH:mm DD.MM.YYYY")}</td>
                            </tr>
                            <tr>
                                <td>Hodnocení:</td>
                                <td>
                                    <StarRatings rating={Number(otherArticleInfo.rating)}
                                                       starRatedColor={"rgb(18,179,0)"}
                                                       numberOfStars={5}
                                                       starDimension={"15px"}
                                                       starSpacing={"0px"}/>
                                </td>
                            </tr>
                            <tr>
                                <td>Celkový počet hodnocení:</td>
                                <td>{otherArticleInfo.countOfAllRatings}</td>
                            </tr>
                            <tr>
                                <td>Celkový počet komentářů:</td>
                                <td>{otherArticleInfo.countOfAllComments}</td>
                            </tr>
                            <tr>
                                <td>Počet uložení článku:</td>
                                <td>{otherArticleInfo.countOfSaved}</td>
                            </tr>
                            <tr className={"text-center"}>
                                <td><Link to={"/clanek/"+articleId}>Zobrazit článek</Link></td>
                                <td><Link to={"/admin/upravit-clanek/"+articleId}>Upravit článek</Link></td>
                            </tr>
                            </tbody>
                        </Table>
                    </Col>
                </Row>
                {isSurveyExists ?
                <Row className={"align-items-center"}>
                    <h3 className={"text-center"}>Anketa: {surveyText}</h3>
                </Row>
                    : ""}
                {isSurveyExists ?
                <Row className={"admin-stats-row"}>
                    <Col style={{maxHeight:"60vh"}}>
                        <Chart type={"pie"} data={{
                            labels: surveyOptionsWithVotes.map(opt => opt.text),
                            datasets: [
                                {
                                    data: surveyOptionsWithVotes.map(opt => opt.count)
                                }
                            ]
                        }}/>
                    </Col>
                    <Col>
                        <Table responsive>
                            <thead>
                            <tr>
                                <th colSpan={4} className={"text-center"}>Anketa údaje</th>
                            </tr>
                            </thead>
                            <tbody>
                            <tr>
                                <td>Název ankety:</td>
                                <td colSpan={3}>{surveyText}</td>
                            </tr>
                            <tr>
                                <td colSpan={1}>Počet hlasů celkem:</td>
                                <td colSpan={3}>{surveyCountAllVotes}</td>
                            </tr>
                            <tr>
                                <td>Pozice</td>
                                <td>Text možnosti</td>
                                <td>Počet odpovědí</td>
                                <td>%</td>
                            </tr>
                            {surveyOptionsWithVotes.map((option, index) =>{
                                return (
                                    <tr key={index}>
                                        <td>{option.position}</td>
                                        <td>{option.text}</td>
                                        <td>{option.count}</td>
                                        <td>{((option.count / surveyCountAllVotes) *100)}%</td>
                                    </tr>
                                )
                            })}
                            </tbody>
                        </Table>
                    </Col>
                </Row>
                    : ""}
            </Container>
        )
}

export default SpecificArticleStatistics;