import React, {useEffect, useState} from "react";
import {Chart} from "react-chartjs-2";
import {getDatesBetweenFromRange} from "../../../components/DateHelper";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import {Col, Container, Row, Spinner, Table} from "react-bootstrap";
import StarRatings from "react-star-ratings/build/star-ratings";
import moment from "moment";
import {getGender} from "../../../components/Constans";
import {Link} from "react-router-dom";

function SpecificUserStatistics({dateRange, userId}) {

    const [labels, setLabels] = useState(getDatesBetweenFromRange(dateRange));

    const [userArticleCommentsDataset, setUserArticleCommentsDataset] = useState({
        label: "Nové komentáře od uživatele",
        data: [],
        borderWidth: 1,
        borderColor: "#0000BB",
        backgroundColor: "#0000CC"
    });
    const [userArticleRatingsDataset, setUserArticleRatingsDataset] = useState({
        label: "Nové hodnocení od uživatele",
        data: [],
        borderWidth: 1,
        borderColor: "#00BBBB",
        backgroundColor: "#00CCCC"
    });
    const [userArticleReadsDataset, setUserArticleReadsDataset] = useState({
        label: "Nové zobrazení článků.",
        data: [],
        borderWidth: 1,
        borderColor: "#008888",
        backgroundColor: "#007777"
    });
    const [userUniqueLoginsDataset, setUserUniqueLoginsDataset] = useState({
        label: "Unikátní přihlášení",
        data: [],
        borderWidth: 1,
        borderColor: "#00BB00",
        backgroundColor: "#00CC00"
    });
    const [allComments, setAllComments] = useState(0);
    const [allRatings, setAllRatings] = useState(0);
    const [otherUserInfo, setOtherUserInfo] = useState({
        age: 0,
        averageRating: 0,
        email: "",
        username: "",
        registrationDateTime: "",
        birthDate: "",
        gender: "",
        roles: "",
        countOfSavedArticles: 0
    });
    const [loading, setLoading] = useState(true);

    useEffect(() => {
        setLoading(true);
        let tempConfig = {...AxiosConfig()};
        tempConfig.params = {
            startDate: moment(dateRange.start).toISOString(),
            endDate: moment(dateRange.end).toISOString()
        };
        axios.get(BASE_URL + "/admin/statistics/user/" + userId, tempConfig).then(res => {
            setUserArticleCommentsDataset(prevState => ({
                ...prevState,
                data: res.data.body.articleComments.map(a => a.count)
            }));
            setUserArticleRatingsDataset(prevState => ({
                ...prevState,
                data: res.data.body.articleRatings.map(a => a.count)
            }));
            setUserArticleReadsDataset(prevState => ({
                ...prevState,
                data: res.data.body.articleReads.map(a => a.count)
            }));
            setUserUniqueLoginsDataset(prevState => ({
                ...prevState,
                data: res.data.body.uniqueLogins.map(a => a.count)
            }));
            //setLabels(getDatesBetweenFromRange(dateRange));
            setLabels(res.data.body.uniqueLogins.map(a => moment(a.date).format("DD.MM.YYYY")));
            setAllComments(res.data.body.countAllComments);
            setAllRatings(res.data.body.countAllRatings);
            setOtherUserInfo({
                age: res.data.body.age,
                averageRating: res.data.body.averageOfAllRatings,
                email: res.data.body.email,
                username: res.data.body.username,
                registrationDateTime: res.data.body.registrationDateTime,
                birthDate: res.data.body.birthDate,
                gender: res.data.body.gender,
                roles: res.data.body.roles.join(", "),
                countOfSavedArticles: res.data.body.countOfSavedArticles
            });
            setLoading(false);
        }).catch(err => {
            console.log(err);
        })
    }, [dateRange, userId]);

    if(loading)
        return(
            <Row className={"text-center justify-content-center"}>
                <br/>
                <Spinner />
                <h3>Probíhá načítání</h3>
            </Row>
        )
    if(!loading)
    return (
        <Container>
            <Row>
                <h3 className={"text-center"}>Informace o uživateli: {otherUserInfo.username}</h3>
            </Row>
            <Row>
                <Chart type={"line"} data={{
                    labels: labels,
                    datasets: [
                        userArticleCommentsDataset,
                        userArticleRatingsDataset,
                        userArticleReadsDataset,
                        userUniqueLoginsDataset]
                }}/>
            </Row>
            <Row className={"mt-3"}>
                <Col>
                    <Chart type={"pie"} data={{
                        labels: ["Celkový počet komentářů", "Celkový počet hodnocení"],
                        datasets: [
                            {
                                data: [allComments, allRatings],
                                borderColor: ["#0000BB", "#00BBBB"],
                                backgroundColor: ["#0000CC", "#00CCCC"]
                            },
                        ]
                    }}/>
                </Col>
                <Col>
                    <Table responsive>
                        <thead>
                        <tr>
                            <th className={"text-center"} colSpan={2}>Ostatní údaje</th>
                        </tr>
                        </thead>
                        <tbody>
                        <tr>
                            <td>Datum narození:</td>
                            <td>{moment(otherUserInfo.birthDate).format("DD.MM.YYYY")}</td>
                        </tr>
                        <tr>
                            <td>Věk:</td>
                            <td>{otherUserInfo.age} let</td>
                        </tr>
                        <tr>
                            <td>Průměrné hodnocení:</td>
                            <td>
                                <StarRatings rating={Number(otherUserInfo.averageRating)}
                                             starRatedColor={"rgb(18,179,0)"}
                                             numberOfStars={5}
                                             starDimension={"15px"}
                                             starSpacing={"0px"}/>
                            </td>
                        </tr>
                        <tr>
                            <td>Email:</td>
                            <td>{otherUserInfo.email}</td>
                        </tr>
                        <tr>
                            <td>Registrován dne:</td>
                            <td>{moment(otherUserInfo.registrationDateTime).format("HH:mm DD.MM.YYYY")}</td>
                        </tr>
                        <tr>
                            <td>Role:</td>
                            <td>{otherUserInfo.roles}</td>
                        </tr>
                        <tr>
                            <td>Pohlaví:</td>
                            <td>{getGender(otherUserInfo.gender)}</td>
                        </tr>
                        <tr>
                            <td>Počet uložených článků:</td>
                            <td>{otherUserInfo.countOfSavedArticles}</td>
                        </tr>
                        <tr className={"text-center"}>
                            <td colSpan={4}>
                                <Link to={"/uzivatel/"+userId}>Zobrazit uživatele</Link>
                            </td>
                        </tr>
                        </tbody>
                    </Table>
                </Col>
            </Row>
        </Container>
    )
}

export default SpecificUserStatistics;