import React, {useState} from "react";
import {useParams} from "react-router";
import CustomDateRangeSelect from "../../viewComponents/CustomDateRangeSelect";
import SpecificUserStatistics from "./SpecificUserStatistics";
import {Button, Form, Row, Table} from "react-bootstrap";
import {Link} from "react-router-dom";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import axios from "axios";

function UserStatistics() {

    const [userId, setUserId] = useState(useParams()["userId"]);
    const [searchText, setSearchText] = useState("");
    const [dateRange, setDateRange] = useState({end: undefined, start: undefined});
    const [searchedUsers, setSearchedUsers] = useState([]);

    function search(e){
        e.preventDefault();
        let tempConfig = {...AxiosConfig()};
        tempConfig.params = {search: searchText};
        axios.get(BASE_URL + "/admin/statistics/user", tempConfig).then(res => {
            setSearchedUsers(res.data.body);
        }).catch(err => {
           console.log(err);
        });
    }

    return (
        <div id={"content"}>
            {userId
                ? <>
                    <br/>
                    <Row className={"justify-content-end"}>
                        <Button variant={"warning"}
                                style={{width: "auto"}}
                                as={Link}
                                to={"/admin/statistiky/uzivatel"}
                                onClick={() => setUserId(undefined)}>
                            Zpět na výběr uživatelů
                        </Button>
                    </Row>
                    <CustomDateRangeSelect onConfirmRange={(range) => setDateRange(range)}/>
                    <SpecificUserStatistics userId={userId} dateRange={dateRange}/>
                </>
                : <>
                    <br/>
                    <Form onSubmit={search}>
                    <Row className={"flex-row align-items-center justify-content-center"}>
                        <Form.Label className={"w-auto"}>Vyhledat uživatele:</Form.Label>
                        <Form.Control className={"w-50"}
                                      type={"text"}
                                      value={searchText}
                                      onChange={(e) => setSearchText(e.target.value)}/>
                        <Button className={"w-auto"}
                                variant={"outline-success"}
                                type={"submit"}>
                            Vyhledat
                        </Button>
                    </Row>
                    </Form>
                    <br/>
                    <Table responsive>
                        <tbody>
                    {Array.isArray(searchedUsers) && searchedUsers.length > 0
                        ? searchedUsers.map((user, index) => {
                            return (
                                <tr key={index}>
                                    <td>{Number(index + 1)}</td>
                                    <td>{user.username}</td>
                                    <td>
                                        <Link to={"/admin/statistiky/uzivatel/"+user.id}
                                              onClick={() => setUserId(user.id)}>
                                            Zobrazit statistiky uživatele
                                        </Link>
                                    </td>
                                </tr>
                            )
                        })
                        : <tr>
                            <td colSpan={4} className={"text-center"}>Nebylo nic nalezeno</td>
                        </tr>
                    }
                        </tbody>
                    </Table>
                </>}
        </div>
    )
}

export default UserStatistics;