import React ,{useState, useEffect} from "react";
import {Chart} from "react-chartjs-2";
import {getDatesBetweenFromRange} from "../../../components/DateHelper";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import moment from "moment";

function UsersChart({dateRange}) {

    const [labels, setLabels] = useState(getDatesBetweenFromRange(dateRange));
    const [usersLoginsUniqueDataset, setUsersLoginsUniqueDatasets] = useState({
        label: "Počet unikátních přihlášení",
        data: [],
        borderWidth: 1,
        borderColor: "#00AA00",
        backgroundColor:"#00FF00"
    });
    const [usersLoginsDataset, setUsersLoginsDatasets] = useState({
        label: "Počet přihlášení",
        data: [],
        borderWidth: 1,
        borderColor: "#006600",
        backgroundColor:"#009900"
    });
    const [usersRegistrationDataset, setUsersRegistrationDataset] = useState({
        label: "Počet registrací",
        data: [],
        borderWidth: 1,
        borderColor: "#880000",
        backgroundColor:"#AA0000"
    });


    useEffect(() => {
        let tempConfig = {...AxiosConfig()};
        tempConfig.params = {
            startDate: moment(dateRange.start).toISOString(),
            endDate: moment(dateRange.end).toISOString()
        };
        axios.get(BASE_URL + "/admin/statistics/user/logins", tempConfig).then(res => {
            let newState = {...usersLoginsUniqueDataset};
            newState.data = res.data.body.map(o => o.count);
            setLabels(res.data.body.map(o => moment(o.date).format("DD.MM.YYYY")));
            setUsersLoginsUniqueDatasets(newState);
        }).catch(err => {
            console.log(err);
        });
        axios.get(BASE_URL + "/admin/statistics/user/registrations", tempConfig).then(res => {
            let newState = {...usersRegistrationDataset};
            newState.data = res.data.body.map(o => o.count);
            setLabels(res.data.body.map(o => moment(o.date).format("DD.MM.YYYY")));
            setUsersRegistrationDataset(newState);
        }).catch(err => {
            console.log(err);
        });
        tempConfig.params.unique = false;
        axios.get(BASE_URL + "/admin/statistics/user/logins", tempConfig).then(res => {
            let newState = {...usersLoginsDataset};
            newState.data = res.data.body.map(o => o.count);
            setLabels(res.data.body.map(o => moment(o.date).format("DD.MM.YYYY")));
            setUsersLoginsDatasets(newState);
        }).catch(err => {
            console.log(err);
        });
    }, [dateRange]);

    return (
        <Chart type={"line"} data={{
            labels: labels,
            datasets: [usersLoginsUniqueDataset, usersRegistrationDataset, usersLoginsDataset]
        }}/>
    )
}

export default UsersChart;