import React, {useState, useEffect} from "react";
import {Button, Form, OverlayTrigger, Stack, Table, Tooltip} from "react-bootstrap";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../../components/AxiosConfig";
import CustomModal, {CustomModalTypes} from "../../viewComponents/CustomModal";
import {formatDateTimeWithSettings, setWebTitle} from "../../../components/SettingsManager";
import CustomPagination from "../../viewComponents/CustomPagination";
import EditRolesModal from "../../viewComponents/EditRolesModal";
import {useCookies} from "react-cookie";
import {userIdCookieName} from "../../../components/AuthManager";
import {Link} from "react-router-dom";

function UsersListAdmin() {

    const [users, setUsers] = useState([{
        id:null,
        username: "",
        email: "",
        createTime: null,
        lastLoginTime: null,
        ratingsCount: null,
        commentsCount: null,
        roles: ""
    }]);
    const [cookies] = useCookies();

    const [perPage, setPerPage] = useState(10);
    const [currentPage, setCurrentPage] = useState(1);
    const [modalData, setModalData] = useState({
        header: "",
        text:"",
        type: null,
        show: false
    });
    const [editRolesModalData, setEditRolesModalData] = useState({
        header: "",
        roles: [],
        id: undefined
    });
    const [showEditRolesModal, setShowEditRolesModal] = useState(false);
    const [searchText, setSearchText] = useState("");

    useEffect(() => {
        setWebTitle("Admin uživatelé", true);
        loadUsers();
    }, []);

    function loadUsers(){
        setSearchText("");
        axios.get(BASE_URL + "/admin/user-management/user", AxiosConfig()).then(res => {
            setUsers(res.data.body);
        }).catch(err => {
            if(err.response.data.body){
                setModalData({
                    header: "Chyba",
                    text: err.response.data.body.map(e=>e.message).join(" "),
                    type: CustomModalTypes.Danger,
                    show: true
                });
            }
        });
    }


    function sortTable(e) {
        const attributeName = e.currentTarget.getAttribute("name");
        let element = document.getElementById("icon-" + attributeName);
        let order = element.classList.contains("fa-caret-up") ? 1 : -1;
        //console.log("LOG: Sorting " + attributeName + " with order " + order + " " + (order === 1 ? "UP" : "DOWN"));
        element.classList.contains("fa-caret-up")
            ? element.classList.replace("fa-caret-up", "fa-caret-down")
            : element.classList.replace("fa-caret-down", "fa-caret-up");

        let tempData = [...users];
        tempData = tempData.sort();
        tempData = tempData.sort((a, b) => {
            if (attributeName.toLowerCase().includes("time")) {
                return (
                    (order > 0)
                        ? (new Date(a[attributeName]) - new Date(b[attributeName]))
                        : (new Date(b[attributeName]) - new Date(a[attributeName]))
                );
            } else if (typeof a[attributeName] === "number") {
                return (
                    (order > 0)
                        ? b[attributeName] - a[attributeName]
                        : a[attributeName] - b[attributeName]
                );
            } else if (typeof a[attributeName] === "string") {
                return (
                    (order < 0)
                        ? a[attributeName].localeCompare(b[attributeName])
                        : b[attributeName].localeCompare(a[attributeName])
                );
            } else if (Array.isArray(a[attributeName])){
                if(a[attributeName].length > 1)
                    a[attributeName] = a[attributeName].sort();
                if(b[attributeName].length > 1)
                    b[attributeName] = b[attributeName].sort();
                return (
                    (order < 0)
                        ? a[attributeName].join(" ").localeCompare(b[attributeName].join(" "))
                        : b[attributeName].join(" ").localeCompare(a[attributeName].join(" "))
                );
            }
        });
        setUsers(tempData);
    }

    function changeRoleClicked(userId) {
        if(Number(userId) === Number(cookies[userIdCookieName])){
            setModalData({
                header: "Změna role",
                text:"Nelze změnit vlastní role.",
                type: CustomModalTypes.Warning,
                show: true
            });
        } else {
            let clickedUser = users.find(value => value.id === userId);
            setEditRolesModalData({
                header: "Změna role: " + clickedUser.username,
                roles: clickedUser.roles,
                id: userId,
            });
            setShowEditRolesModal(true);
        }
    }

    function confirmRoleChange(roles){
        axios.put(BASE_URL + "/admin/user-management/user/"+editRolesModalData.id+"/role", JSON.stringify(roles), AxiosConfig()).then(res => {
            setShowEditRolesModal(false);
            setModalData({
                header: "Změna role",
                text: res.data.body,
                type: CustomModalTypes.Success,
                show: true
            });
            loadUsers();
        }).catch(err => {
            if(err.response.data.body){
                setModalData({
                    header: "Chyba",
                    text: err.response.data.body.map(e=>e.message).join(" "),
                    type: CustomModalTypes.Danger,
                    show: true
                });
            }
        });
    }

    return(
        <div id={"content"}>
            <CustomModal modalData={modalData} showModal={modalData.show} onClose={() => setModalData({show: false})}/>
            <EditRolesModal data={editRolesModalData}
                            show={showEditRolesModal}
                            onClose={() => setShowEditRolesModal(false)}
                            onConfirm={confirmRoleChange}/>
            <br/>
            <Stack direction={"horizontal"} gap={2} className={"admin-content-list"}>
                <Button className={"ml-auto"}
                        onClick={loadUsers}
                        variant={"outline-dark"}>Obnovit <i className={"fa-solid fa-rotate"}> </i></Button>
                <Form.Label>Vyhledat:</Form.Label>
                <Form.Control type={"text"}
                              className={"w-auto"}
                              placeholder={"Vyhledávání"}
                              onChange={(e) => setSearchText(e.target.value)}
                              value={searchText}/>
                <Form.Label className={"ms-auto"}>Zobrazení na stránce</Form.Label>
                <Form.Select className={"w-auto"} onChange={(e) => {
                    setPerPage(e.target.value);
                }}>
                    <option value={10}>10</option>
                    <option value={20}>20</option>
                    <option value={40}>40</option>
                    <option value={60}>60</option>
                </Form.Select>
            </Stack>
            <Table striped bordered responsive>
                <thead>
                <tr>
                    <th>#</th>
                    <th name={"id"}>ID</th>
                    <th name={"username"} onClick={sortTable}>Jméno
                        <i id={"icon-username"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"email"} onClick={sortTable}>Email
                        <i id={"icon-email"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"createTime"} onClick={sortTable}>Registrován
                        <i id={"icon-createTime"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"lastLoginTime"} onClick={sortTable}>Poslední přihlášení
                        <i id={"icon-lastLoginTime"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"ratingsCount"} onClick={sortTable}>Počet <br/>hodnocení
                        <i id={"icon-ratingsCount"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"commentsCount"} onClick={sortTable}>Počet <br/>komentářů
                        <i id={"icon-commentsCount"} style={{float: "right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th name={"roles"} onClick={sortTable}>Role
                        <i id={"icon-roles"} style={{float:"right"}} className="fa-solid fa-caret-up"> </i>
                    </th>
                    <th>
                        Statistiky
                    </th>
                </tr>
                </thead>
                <tbody>
                {users ? users.filter(u => {return u.username.toLowerCase().includes(searchText.toLowerCase())})
                    .slice(((currentPage - 1) * perPage), (currentPage * perPage))
                    .map((user, index) => {
                    return(
                        <tr key={index}>
                            <td>{(index + 1) + ((currentPage - 1) * perPage)}</td>
                            <td>{user.id}</td>
                            <td> <a href={"/uzivatel/"+user.id}>{user.username}</a></td>
                            <td>{user.email}</td>
                            <td>{formatDateTimeWithSettings(user.createTime)}</td>
                            <td>
                                {user.lastLoginTime ? formatDateTimeWithSettings(user.lastLoginTime): "Nebyl přihlášen"}
                            </td>
                            <td>{user.ratingsCount}</td>
                            <td>{user.commentsCount}</td>
                            <td>
                                {user.roles.length > 1 ? user.roles.join(",") : user.roles}
                                <OverlayTrigger overlay={<Tooltip>Klikni pro změnu rolí</Tooltip>}>
                                <i  style={{cursor: "pointer", color:"var(--bs-link-color)"}}
                                    onClick={() => changeRoleClicked(user.id)}
                                    className="fa-solid fa-pen-to-square float-end"/>
                                </OverlayTrigger>
                            </td>
                            <td>
                                <Link to={"/admin/statistiky/uzivatel/"+user.id}>Zobrazit</Link>
                            </td>
                        </tr>
                    )}) : ""}
                </tbody>
                <tfoot>
                <tr>
                    <td colSpan={10} className={"text-center"}>
                        <CustomPagination defaultPage={1}
                                          maxItemsPerPage={perPage}
                                          numOfItems={users.filter(u => {return u.username.toLowerCase().includes(searchText.toLowerCase())}).length}
                                          reloadDependency={users}
                                          onChange={(e) => setCurrentPage(e)}/>
                    </td>
                </tr>
                </tfoot>
            </Table>
        </div>
    )
}

export default UsersListAdmin;