import axios from 'axios';
import {BASE_URL, AxiosConfig} from "../../components/AxiosConfig";
import React, {useContext, useEffect, useState} from 'react';
import {Button, Card, InputGroup, Form, Spinner, Col, Row, Dropdown, DropdownButton} from 'react-bootstrap';
import {useParams} from "react-router";
import {useCookies} from "react-cookie";
import StarRatings from "react-star-ratings/build/star-ratings";
import CustomModal, {CustomModalTypes, CustomYesNoModal} from "../viewComponents/CustomModal";
import {ARTICLE_IMAGE_URL, USER_IMAGE_URL} from "../../components/Constans";
import parse from 'html-react-parser';
import {authTokenExists, userIdCookieName, userRolesExists} from "../../components/AuthManager";
import {Link, useNavigate} from "react-router-dom";
import {formatDateTimeWithSettings, SettingsContext, setWebTitle} from "../../components/SettingsManager";
import CustomPagination from "../viewComponents/CustomPagination";
import SEOManager, {SEO_ROBOTS} from "../../components/SEOManager";

function ShowArticle() {
    const articleId = useParams().articleId;
    const [cookies] = useCookies(userIdCookieName);
    const navigate = useNavigate();
    const maxOfComments = 8;
    const [currentCommentPage, setCurrentCommentPage] = useState(1);
    const [commentsSortOrder, setCommentsSortOrder] = useState(1);
    const settings = useContext(SettingsContext);

    const [articleData, setArticleData] = useState({
        text: "",
        title: "",
        description: "",
        createTime: null,
        lastUpdateTime: null,
        viewsCount: undefined
    });
    const [loadingArticleData, setLoadingArticleData] = useState(true);
    const [commentsData, setCommentsData] = useState([{
        id: null,
        comment: "",
        createTime: null,
        userId: null,
        username: "",
        incorrect:undefined
    }]);
    const [loadComments, setLoadComments] = useState(true);

    const [modalData, setModalData] = useState({
        show: false,
        header: "",
        text: "",
        type: "success"
    });
    const [yesNoModalData, setYesNoModalData] = useState({
        show:false,
        header: "",
        text: ""
    });

    const [loadRatings, setLoadRatings] = useState(true);
    const [ratingData, setRatingData] = useState({
        rating: 0,
        count: 0
    });

    const [userRatingData, setUserRatingData] = useState({
        rating: null,
        createTime: null
    });

    const [userListButtonDisable, setUserListButtonDisable] = useState(true);
    const [userSavedArticleId, setUserSavedArticleId] = useState(undefined);

    const [surveyData, setSurveyData] = useState({
        surveyText: undefined,
        surveyVisibleResults: undefined,
        surveyOptions: [
            {position: 1, text: "", count: undefined}
        ]
    });
    const [surveyAllVotes, setSurveyAllVotes] = useState(undefined);
    const [surveySelectedVotePosition, setSurveySelectedVotePosition] = useState(null);

    useEffect(() => {
        setWebTitle(articleData.title, true);
    }, [articleData]);

    useEffect(() => {
            axios.get(BASE_URL + '/content/article/' + articleId, AxiosConfig()).then(res => {
                setArticleData(res.data.body);
                setLoadingArticleData(false);
                if (res.data.body.survey) {
                    setSurveyData({...res.data.body.survey});
                    let countVotes = 0;
                    res.data.body.survey.surveyOptions.forEach(op => countVotes+=op.count);
                    setSurveyAllVotes(countVotes);
                } else {
                    setSurveyData(undefined);
                }
            }).catch(err => {
                let errors = err.response.data.errors;
                if (errors.map(error => {
                    return error.key
                }).includes("article-is-not-visible")) {
                    setModalData({
                        show: true,
                        header: "Chyba",
                        text: errors.map(e => {
                            return e.message
                        }).join(" "),
                        type: CustomModalTypes.Danger,
                        redirect: -1
                    });
                }
            });

    }, [articleId]);

    useEffect(() => {
        if(authTokenExists() && surveyData && surveyData.surveyText){
            axios.get(BASE_URL + "/content/article/"+articleId+"/survey/vote", AxiosConfig()).then(res => {
                setSurveySelectedVotePosition(res.data.body);
                document.getElementById("survey-option-position-"+res.data.body).checked = true;
            }).catch(err => {

            });
        }
    }, [surveyData]);

    useEffect(() => {
            setLoadComments(false);
            // get data comments
            axios.get(BASE_URL + '/content/article/' + articleId + '/comments', AxiosConfig()).then(res => {
                setCommentsData(sortComments(commentsSortOrder, res.data.body));
            }).catch(err => {
                if (err.response.data.errors) {
                    setModalData({
                        header: "Chyba",
                        text: err.response.data.errors.map(e => e.message).join(" "),
                        show: true,
                        type: CustomModalTypes.Danger
                    });
                }
            });

    }, [articleId, loadComments]);

    useEffect(() => {
            setLoadRatings(false);
            axios.get(BASE_URL + '/content/article/' + articleId + '/rating', AxiosConfig()).then(res => {
                setRatingData(res.data.body);
            }).catch(err => {
                if (err.response.data.errors) {
                    setModalData({
                        header: "Chyba",
                        text: err.response.data.errors.map(e => e.message).join(" "),
                        show: true,
                        type: CustomModalTypes.Danger
                    });
                }
            });
            if (authTokenExists()) {
                axios.get(BASE_URL + '/content/article/' + articleId + '/rating/user', AxiosConfig()).then(res => {
                    setUserRatingData(res.data.body);
                }).catch(err => {
                    setUserRatingData({rating: null, createTime: null});
                    if (err.response.data.errors.length !== 1 && err.response.data.errors[0].key !== "not-rating-from-user")
                        if (err.response.data.errors) {
                            setModalData({
                                header: "Chyba",
                                text: err.response.data.errors.map(e => e.message).join(" "),
                                show: true,
                                type: CustomModalTypes.Danger
                            });
                        }
                });
            } else {
                setUserRatingData({rating: null, createTime: null});
            }

    }, [articleId, loadRatings]);


    function addComment() {
        let commentText = document.getElementById("commentAddText").value;
        if (commentText) {
            let data = JSON.stringify({comment: commentText})
            axios.post(BASE_URL + "/content/article/" + articleId + "/comment", data, AxiosConfig()).then(res => {
                setModalData({
                    show: true,
                    header: "Přidání komentáře.",
                    text: res.data.body,
                    type: CustomModalTypes.Success
                });
                setLoadComments(true);
            }).catch(err => {
                if (err.response.data.errors) {
                    setModalData({
                        show: true,
                        header: "Chyba při přidání komentáře.",
                        text: err.response.data.errors.map(e => e.message).join("<br>"),
                        type: CustomModalTypes.Danger
                    });
                }
            });
            document.getElementById("commentAddText").value = "";
        }
    }

    useEffect(() => {
        if (cookies[userIdCookieName]) {
            axios.get(BASE_URL + "/user-management/user/article/" + articleId + "/saved", AxiosConfig()).then(res => {
                setUserSavedArticleId(res.data.body.id);
                setUserListButtonDisable(false);
            }).catch(err => {
                setUserListButtonDisable(false);
            });
        }
    }, [userSavedArticleId,articleId]);

    function closeModal() {
        if (!!modalData.redirect)
            navigate(modalData.redirect);
        setModalData({show: false});
    }

    function deleteComment(e) {
        let deleteId = e.target.attributes.commentid.value;
        axios.delete(BASE_URL + '/content/article/comment/' + deleteId, AxiosConfig()).then(res => {
            setModalData({
                show: true,
                header: "Smazání komentáře",
                text: res.data.body,
                type: "success"
            });
            setLoadComments(true);
        }).catch(err => {
            if (err.response.data.errors) {
                setModalData({
                    header: "Chyba",
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    show: true,
                    type: CustomModalTypes.Danger
                });
            }
        });
    }

    function changeRating(rating) {
        let data = ({rating});
        axios.post(BASE_URL + '/content/article/' + articleId + '/rating', data, AxiosConfig()).then(res => {
            setModalData({show: true, header: "Změna hodnocení.", type: "success", text: res.data.body});
            setLoadRatings(true);
        }).catch(err => {
            if (err.response.data.errors) {
                setModalData({
                    show: true,
                    header: "Přidání hodnocení",
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    type: CustomModalTypes.Danger
                })
            }
        });
    }

    function deleteRating() {
        axios.delete(BASE_URL + '/content/article/' + articleId + '/rating/user', AxiosConfig()).then(res => {
            setModalData({show: true, header: "Odebrání hodnocení.", type: "success", text: res.data.body});
            setLoadRatings(true);
        }).catch(err => {
            if (err.response.data.errors) {
                setModalData({
                    header: "Chyba",
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    show: true,
                    type: CustomModalTypes.Danger
                });
            }
        });
    }

    function sortComments(order, comments) {
        let tempComments = [...commentsData];
        if (comments)
            tempComments = [...comments];
        tempComments.sort(function (a, b) {
            return (order > 0)
                ? (new Date(b.createTime) - new Date(a.createTime))
                : (new Date(a.createTime) - new Date(b.createTime));
        });
        return tempComments;
    }

    function changeSortComments(e) {
        let order = Number(e.target.value);
        setCommentsData(sortComments(order));
        setCommentsSortOrder(order);
    }

    function handleListButton() {
        setUserListButtonDisable(true);
        if (Number(userSavedArticleId)) {
            axios.delete(BASE_URL + "/user-management/user/article/saved/" + userSavedArticleId, AxiosConfig()).then(res => {
                setModalData({
                    show: true,
                    type: CustomModalTypes.Success,
                    text: res.data.body,
                    header: "Odebrání ze seznamu"
                });
                setUserSavedArticleId(res.data.body);
            }).catch(err => {
                if (err.response.data.errors) {
                    setModalData({
                        show: true,
                        type: CustomModalTypes.Danger,
                        text: err.response.data.errors.map(e => e.message).join(" "),
                        header: "Odebrání ze seznamu"
                    });
                }
                setUserSavedArticleId(undefined);
            });
        } else {
            axios.post(BASE_URL + "/user-management/user/article/" + articleId + "/saved", null, AxiosConfig()).then(res => {
                setModalData({
                    show: true,
                    type: CustomModalTypes.Success,
                    text: res.data.body,
                    header: "Uložení do seznamu"
                });
                setUserSavedArticleId(res.data.body);
            }).catch(err => {
                if (err.response.data.errors) {
                    setModalData({
                        show: true,
                        type: CustomModalTypes.Danger,
                        text: err.response.data.errors.map(e => e.message).join(" "),
                        header: "Uložení do seznamu"
                    });
                }
                setUserSavedArticleId(undefined);
            });
        }
    }

    function handleSurveyVoteChange(position){
        setSurveySelectedVotePosition(position);
    }

    function confirmSurveyVote(){
        let dataToSend = {position: surveySelectedVotePosition};
        axios.post(BASE_URL + "/content/article/"+articleId+"/survey/vote", JSON.stringify(dataToSend), AxiosConfig()).then(res =>{
            setModalData({
                show: true,
                type: CustomModalTypes.Success,
                text: res.data.body,
                header: "Hlasování v anketě"
            });
        }).catch(err => {
            if (err.response.data.errors) {
                setModalData({
                    show: true,
                    type: CustomModalTypes.Danger,
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    header: "Hlasování v anketě"
                });
            }
        });
    }

    function setCommentIncorrect(e){
        let commentId = Number(e.target.attributes.commentid.value);
        let incorrect = commentsData.find(comment => commentId === comment.id).incorrect;
        setYesNoModalData({
            show:true,
            text: "Chcete tento komentář skrýt jako nevhodný? Komentář bude možné i nadále zobrazit, bude nutné však zobrazení potvrdit.",
            header: "Skrytí nevhodného komentáře",
            commentId: commentId,
            incorrect: incorrect
        });
    }

    function confirmIncorrectComment(){
        let commentId = yesNoModalData.commentId;
        let incorrect = yesNoModalData.incorrect;
        setYesNoModalData({show:false});
        axios.put(BASE_URL + "/content/article/comment/"+commentId+"/incorrect", (!incorrect), AxiosConfig()).then(res => {
            setLoadComments(true);
            setModalData({
                show: true,
                type: CustomModalTypes.Success,
                text: res.data.body,
                header: "Úprava viditelnosti komentáře."
            });
        }).catch(err => {
            if(err.response.data.errors){
                setModalData({
                    show:true,
                    type: CustomModalTypes.Danger,
                    text: err.response.data.errors.map(e => e.message).join(" "),
                    header: "Úprava viditelnosti komentáře."
                })
            }
        })
    }

    return (
        <div id='content'>
            <SEOManager description={articleData.description}
                        robots={SEO_ROBOTS.INDEX_FOLLOW}/>
            {/* Modals for information */}
            <CustomModal modalData={modalData} showModal={modalData.show} onClose={closeModal}/>
            <CustomYesNoModal modalData={yesNoModalData}
                              showModal={yesNoModalData.show}
                              onConfirm={confirmIncorrectComment}
                              onCancel={()=>setYesNoModalData({show:false})}/>
            {/* Article */}
            {loadingArticleData ? <Spinner/> :
                <Card>
                    {articleData.visibility === false
                        ? <Card.Header className={"text-center"} as={"h3"}>Článek je skrytý.</Card.Header>
                        : ""}
                    <Card.Header>
                        <Row className={"show-article-title-row"}>
                            <Col>
                                <Card.Title as={"h4"}>{articleData.title}</Card.Title>
                            </Col>
                            <Col>
                                {cookies[userIdCookieName] ?
                                    <Card.Title className={"show-article-title-save"}>
                                        <Button
                                            variant={"outline-" + (Number.isInteger(userSavedArticleId) ? "danger" : "success")}
                                            onClick={handleListButton}
                                            disabled={userListButtonDisable}>
                                            {Number.isInteger(userSavedArticleId) ? "Odebrat ze seznamu" : "Uložit do seznamu"}
                                        </Button>
                                    </Card.Title>
                                    : ""}
                            </Col>
                            {userRolesExists("ADMIN")
                                ? <Col className={"text-center"} style={{maxWidth: "fit-content"}}>
                                    <DropdownButton title={"Možnosti admina"} variant={"outline-info"}>
                                        <Dropdown.Item href={"/admin/statistiky/clanek/" + articleId}>
                                            Zobrazit statistiky</Dropdown.Item>
                                        <Dropdown.Item href={"/admin/upravit-clanek/" + articleId}>
                                            Upravit článek</Dropdown.Item>
                                    </DropdownButton>
                                </Col>
                                : ""}
                        </Row>
                    </Card.Header>
                    <Card.Body>
                        <Row className={"show-article-description-row"}>
                            <Col className={"d-flex align-items-center justify-content-center show-article-image"}>
                                <img src={ARTICLE_IMAGE_URL(articleId)}
                                     alt={"Obrázek"} />
                            </Col>
                            <Col>
                                <Card.Subtitle>
                            {articleData.description}
                                </Card.Subtitle>
                            </Col>
                        </Row>
                        <br/>
                        <Card.Text as={"div"} className={"ql-editor"}>{parse(articleData.text)}</Card.Text>
                        {surveyData
                            ? <Card.Text as={"div"}>
                                <Row className={"justify-content-center"}>{surveyData.surveyText}</Row>
                                <Row className={"justify-content-center align-items-center"}>
                                    {surveyData.surveyOptions.map((option, index) => {
                                        return (
                                            <Form.Check key={index}
                                                        className={"d-flex justify-content-center"}
                                                        type={"radio"}
                                                        label={surveyData.surveyVisibleResults
                                                            ? (option.text +  " ( Počet hlasů: "
                                                            + option.count + " | " + ((option.count/surveyAllVotes)*100) + "%)")
                                                            : option.text}
                                                        id={"survey-option-position-"+option.position}
                                                        name={"survey-group"}
                                                        defaultChecked={(option.position === surveySelectedVotePosition)}
                                                        onChange={() => handleSurveyVoteChange(option.position)}
                                                        disabled={!cookies[userIdCookieName]}/>
                                        )
                                    })}
                                    <Button className={"w-auto"}
                                            variant={cookies[userIdCookieName] ? "outline-success" : "danger"}
                                            onClick={confirmSurveyVote}
                                            disabled={!cookies[userIdCookieName]}>
                                        {cookies[userIdCookieName] ? "Hlasovat" : "Musíte být přihlášen"}
                                    </Button>
                                </Row>
                            </Card.Text>
                            : ""}
                    </Card.Body>
                    <Card.Footer>
                        <Row className={"show-article-footer-row"}>
                            <Col className={"t-start"}>
                                {userRatingData.rating === null
                                    ? ""
                                    : <Row>
                                        <span>Vaše hodnocení: {userRatingData.rating}</span>
                                        <span>Ze dne: {formatDateTimeWithSettings(userRatingData.createTime)}</span>
                                    </Row>}
                            </Col>
                            <Col className={"justify-content-center align-items-center d-flex flex-column"}>
                                <Row className={""}>
                                    <StarRatings
                                        rating={ratingData.rating}
                                        starRatedColor={"rgb(18,179,0)"}
                                        numberOfStars={5}
                                        name={"rating"}
                                        changeRating={cookies[userIdCookieName] ? changeRating : undefined}
                                        starDimension={"30px"}
                                        starSpacing={"0px"}/>
                                </Row>
                                {settings.displayNumberOfRatings && ratingData.count ?
                                    <Row>
                                        Počet hodnocení: {ratingData.count}
                                    </Row>
                                    : ""}
                            </Col>
                            <Col className={"show-article-delete-rating align-items-center d-flex"}>
                                {userRatingData.rating === null
                                    ? ""
                                    :
                                    <Button onClick={deleteRating} variant={"outline-danger"}>Smazat hodnocení</Button>}
                            </Col>
                        </Row>
                    </Card.Footer>
                    <Card.Footer>
                        <Row className={"show-article-footer-row"}>
                            <Col className={"t-start"}>
                                {settings.displayUpdateCreateTime !== "update"
                                    ? "Napsáno dne: " + formatDateTimeWithSettings(articleData.createTime)
                                    : ""}
                            </Col>
                            <Col className={"t-center"}>
                                {settings.displayNumberOfViewsArticle && articleData.viewsCount
                                    ? "Počet zobrazení: " + articleData.viewsCount
                                    : ""}
                            </Col>
                            <Col className={"t-end"}>
                                {articleData.createTime !== articleData.lastUpdateTime
                                && settings.displayUpdateCreateTime !== "create"
                                    ? "Upraveno: " + formatDateTimeWithSettings(articleData.lastUpdateTime)
                                    : ""
                                }
                            </Col>
                        </Row>
                    </Card.Footer>
                </Card>
            }
            {/* Comments */}
            {!cookies.userId ? "" :
                <Card>
                    <Card.Header as="h5">Přidat komentář</Card.Header>
                    <Card.Body className={"d-flex flex-column justify-content-center align-items-center"}>
                        <InputGroup defaultValue={""} id="commentAddText" as="textarea"
                                    placeholder="Zde napište komentář"/>
                        <Button variant="outline-success"
                                className={"w-auto"}
                                onClick={addComment}>Přidat komentář</Button>
                    </Card.Body>
                </Card>
            }
            <Card>
                <Row className={"m-2"}>
                    <Col>
                        <h4 className={"text-start"}>Komentáře</h4>
                    </Col>
                    <Col>
                        <Form.Select className={"ms-auto w-auto"} onChange={changeSortComments} value={commentsSortOrder}>
                            <option value={-1}>Od nejstarších</option>
                            <option value={1}>Od nejnovějších</option>
                        </Form.Select>
                    </Col>
                </Row>
                {commentsData.slice(((currentCommentPage - 1) * maxOfComments), (currentCommentPage * maxOfComments)).map(comment => {
                    return (
                        <div key={comment.id} className={"ms-3 me-3 border p-2"}>
                            <Row className={"mb-2"}>
                                <Col style={{maxWidth: "45px"}} className={"me-1"}>
                                    <img src={USER_IMAGE_URL(comment.userId)}
                                         alt={""} height={"40px"} width={"40px"}/>
                                </Col>
                                <Col className={"d-flex align-items-center justify-content-start"}>
                                    <Link to={"/uzivatel/" + comment.userId}>{comment.username}</Link>
                                    <span className={"ps-1"}>komentoval: </span>
                                </Col>
                                <Col className={"text-end d-flex align-items-center justify-content-end"}>
                                    <span>{formatDateTimeWithSettings(comment.createTime)}</span>
                                    {Number(cookies.userId) === Number(comment.userId)
                                        ? <Button commentid={comment.id}
                                                  className={"ms-3"}
                                                  variant={"outline-danger"}
                                                  onClick={deleteComment}>Smazat</Button>
                                        : ""}
                                    {userRolesExists("ADMIN")
                                        ? <Button commentid={comment.id}
                                                  className={"ms-3 w-auto"}
                                                  variant={comment.incorrect ? "success" : "danger"}
                                                  onClick={setCommentIncorrect}>
                                            {comment.incorrect ? "Vhodný" : "Nevhodný"}
                                    </Button>
                                        : ""}
                                </Col>
                            </Row>
                            <Row className={"ms-auto mb-auto me-auto ps-2 border-top"} id={"comment-article-id-"+comment.id}>
                                {comment.incorrect
                                    ? <div><i>Komentář je správcem označen jako nevhodný. </i>
                                        <Link onClick={() => {
                                                    document.getElementById("comment-article-id-"+comment.id).innerText = comment.comment;
                                                }}>
                                            <i>Zobrazit</i>
                                        </Link>
                                    </div>
                                    : comment.comment}
                            </Row>
                        </div>
                    )
                })}
            </Card>
            <CustomPagination maxItemsPerPage={maxOfComments}
                              reloadDependency={commentsData}
                              defaultPage={1}
                              numOfItems={commentsData.length}
                              onChange={(e) => setCurrentCommentPage(e)}/>
        </div>
    );
}

export default ShowArticle;