import axios from 'axios';
import React, {useState, useEffect} from 'react'
import {Form, Button, Stack} from 'react-bootstrap';
import {AxiosConfig, BASE_URL} from '../../components/AxiosConfig';
import {useCookies} from "react-cookie";
import {useNavigate} from "react-router-dom";
import CustomAlert from "../viewComponents/CustomAlert";
import {setAuthToken, setUserRoles, userCookiePathConf, userIdCookieName} from "../../components/AuthManager";
import {setWebTitle} from "../../components/SettingsManager";
import SEOManager, {SEO_ROBOTS} from "../../components/SEOManager";

function Login() {

    const navigate = useNavigate();
    const [cookies, setCookie] = useCookies();
    const [loginFormData, setLoginFormData] = useState({
        email: "",
        password: ""
    });
    const [loginErrors, setLoginErrors] = useState([{key: "", message: ""}]);
    const [hiddenError, setHiddenError] = useState(true);

    useEffect(() => {
        setWebTitle("Registrace", true);
    }, []);

    function onSubmit(e) {
        e.preventDefault();
        axios.post(BASE_URL + '/auth/login', JSON.stringify(loginFormData), AxiosConfig()).then(res => {
            setHiddenError(true);
            setCookie(userIdCookieName, res.data.body.id, userCookiePathConf);
            setAuthToken(res.data.body.token);
            setUserRoles(res.data.body.roles);
            navigate('/uzivatel/'+res.data.body.id);
        }).catch(err => {
            setLoginErrors(err.response.data.errors);
            setHiddenError(false);
        });
    }


    return (
        <div id='content'>
            <SEOManager description={"Přihlášení do systému."}
                        robots={SEO_ROBOTS.INDEX_FOLLOW}/>
            <CustomAlert alertData={{content:loginErrors, header:"Chyba při přihlášení.", variant:"danger"}}
                         display={!hiddenError}
                         onCloseAlert={() => setHiddenError(false)}/>
        <Form onSubmit={onSubmit}>
            <Stack gap={2} className="col-md-5 mx-auto">
            <Form.Group className="mb-3" controlId="formBasicEmail">
                <Form.Label>Email:</Form.Label>
                <Form.Control type="email" placeholder="Zadejte email" value={loginFormData.email}
                              onChange={e => setLoginFormData({
                                  email: e.target.value,
                                  password: loginFormData.password})}/>
            </Form.Group>

            <Form.Group className="mb-3" controlId="formBasicPassword">
                <Form.Label>Heslo</Form.Label>
                <Form.Control type="password" placeholder="Heslo" value={loginFormData.password}
                              onChange={e => setLoginFormData({
                                  email: loginFormData.email,
                                  password: e.target.value})}/>
            </Form.Group>
            <Button variant="primary" type="submit">Přihlásit</Button>
            </Stack>
        </Form>
        </div>
    )
}

export default Login;