import React, {useContext, useEffect, useState} from "react";
import parse from "html-react-parser";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../components/AxiosConfig";
import {SettingsContext} from "../../components/SettingsManager";
import {useNavigate} from "react-router-dom";
import CustomModal, {CustomModalTypes} from "../viewComponents/CustomModal";
import SEOManager, {SEO_ROBOTS} from "../../components/SEOManager";

function AboutAuthor() {

    const [authorPageData, setAuthorPageData] = useState({
        text: "",
        facebookUrl: "",
        instagramUrl: "",
        twitterUrl: "",
        linkedInUrl: "",
        socialNetworksHeading: ""
    });
    const [modalData, setModalData] = useState({});
    const navigate = useNavigate();
    const settings = useContext(SettingsContext);
    useEffect(() => {
        if(!settings.enableAuthorPage)
            navigate(-1);
    }, [settings]);

    useEffect(() => {
        axios.get(BASE_URL + "/settings/author-page", AxiosConfig()).then(res => {
            setAuthorPageData(res.data.body);
        }).catch(err => {
            if(err.response.data.errors){
                setModalData({
                    header: "Chyba",
                    text: err.response.data.errors.map(e=>e.message).join(" "),
                    show: true,
                    type: CustomModalTypes.Danger
                });
            }
        });
    }, []);

    return (
        <div id={"content"}>
            <SEOManager description={"Stránka o autorovi, zjistěte kdo tvoří tento blog."}
                        robots={SEO_ROBOTS.INDEX_FOLLOW}/>
            <CustomModal modalData={modalData} showModal={modalData.show} onClose={()=> setModalData(null)}/>
            <div className={"ql-editor"}>{parse(authorPageData.text)}</div>
            <div className={"mb-5"}>
                <h5>{authorPageData.socialNetworkHeading}</h5>
                {authorPageData.facebookUrl
                    ? <><a href={authorPageData.facebookUrl} target={"_blank"}>
                        <i className={"fa-brands fa-facebook"}/> Facebook</a> <br/></>
                    : ""}
                {authorPageData.instagramUrl
                    ? <><a href={authorPageData.instagramUrl} target={"_blank"}>
                        <i className={"fa-brands fa-instagram"}/> Instagram</a><br/></>
                    : ""}
                {authorPageData.twitterUrl
                    ? <><a href={authorPageData.twitterUrl} target={"_blank"}>
                        <i className={"fa-brands fa-twitter"}/> X / Twitter</a><br/></>
                    : ""}
                {authorPageData.linkedInUrl
                    ? <><a href={authorPageData.linkedInUrl} target={"_blank"}>
                        <i className={"fa-brands fa-linkedin"}/> LinkedIn</a><br/></>
                    : ""}
            </div>
        </div>
    )
}

export default AboutAuthor;