import React, {useContext, useEffect, useState} from "react";
import {useNavigate} from "react-router-dom";
import {SettingsContext} from "../../components/SettingsManager";
import PhotoAlbum from "react-photo-album";
import Lightbox from "yet-another-react-lightbox";
import {Counter, Fullscreen, Slideshow, Thumbnails} from "yet-another-react-lightbox/plugins";
import "yet-another-react-lightbox/plugins/thumbnails.css";
import "yet-another-react-lightbox/styles.css";
import "yet-another-react-lightbox/plugins/counter.css";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../components/AxiosConfig";

function Gallery() {

    const navigate = useNavigate();
    const settings = useContext(SettingsContext);
    const [index, setIndex] = useState(-1);
    const [images, setImages] = useState(undefined);
    const [galleryError, setGalleryError] = useState(undefined);

    useEffect(() => {
        if (!settings.enableAuthorPage)
            navigate(-1);
    }, [settings]);

    useEffect(() => {
        axios.get(BASE_URL + "/settings/gallery/image/list", AxiosConfig()).then(res => {
            setImages([...res.data.body.map(n => {
                return {
                    name: n.name,
                    src: imageUlrFromName(n.name),
                    width: n.width,
                    height: n.height
                }
            })]);
            setGalleryError(undefined);
        }).catch(err => {
            if (err.response.data.errors)
                setGalleryError(err.response.data.errors.map(e => e.message).join(" "));
        });
    }, []);

    function imageUlrFromName(name) {
        return BASE_URL + "/settings/gallery/image/" + name;
    }

    return (
        <div id={"content"}>
            <h1 className={"text-center"}>{settings.galleryPageLinkText ? settings.galleryPageLinkText : "Galerie"}</h1>
            {images
                ? <>
                    <PhotoAlbum photos={images}
                                layout="rows"
                                targetRowHeight={150}
                                onClick={({index}) => setIndex(index)}/>
                    <Lightbox index={index}
                              slides={images}
                              open={index >= 0}
                              close={() => setIndex(-1)}
                              plugins={[Fullscreen, Slideshow, Thumbnails, Counter]}/>
                </>
                : <h3 className={"text-center"}>{galleryError}</h3>}
        </div>
    )
}

export default Gallery;