import {Accordion, Button, Col, Form, Row, Table} from "react-bootstrap";
import React, {useEffect, useState} from "react";
import axios from "axios";
import {AxiosConfig, AxiosConfigImage, BASE_URL} from "../../components/AxiosConfig";
import CustomAlert, {CustomAlertVariants} from "../viewComponents/CustomAlert";
import ImageUpload from "../viewComponents/ImageUpload";
import {formatDateTimeWithSettings, setWebTitle} from "../../components/SettingsManager";
import {useCookies} from "react-cookie";
import {userIdCookieName} from "../../components/AuthManager";
import {useNavigate} from "react-router-dom";
import CustomModal, {CustomModalTypes, CustomYesNoModal} from "../viewComponents/CustomModal";
import moment from "moment";
import {USER_IMAGE_URL} from "../../components/Constans";
import SEOManager, {SEO_ROBOTS} from "../../components/SEOManager";

function EditUser() {

    const [cookies] = useCookies();
    const navigate = useNavigate();
    const [passwordChangeData, setPasswordChangeData] = useState({
        password: "",
        confirmPassword: "",
        oldPassword: ""
    });
    const [alertData, setAlertData] = useState({
        variant: "",
        header: "",
        content: ""
    });
    const [displayAlert, setDisplayAlert] = useState(false);
    const [modalDataYesNo, setModalDataYesNo] = useState({
        header: "",
        text: ""
    });
    const [showModalYesNo, setShowModalYesNo] = useState(false);
    const [modalData, setModalData] = useState({
        header: "",
        text: ""
    });
    const [showModal, setShowModal] = useState(false);

    const [imageData, setImageData] = useState(null);
    const [enableImageSend, setEnableImageSend] = useState(true);

    const [loadData, setLoadData] = useState(true);
    const [privacySettings, setPrivacySettings] = useState({
        emailPrivacy: "PRIVATE",
        registrationTimePrivacy: "LOGGED",
        commentsPrivacy: "PUBLIC",
        ratingsPrivacy: "PUBLIC",
        bioPrivacy: "PUBLIC",
        birthDatePrivacy: "PUBLIC",
        genderPrivacy: "PUBLIC"
    });

    const [userInfo, setUserInfo] = useState({
        createTime: "",
        email: "",
        id: "",
        roles: {},
        username: "",
        bio: "",
        birthDate: "",
        gender: "OTHER"
    });

    useEffect(()=> {
        if(loadData) {
            setLoadData(false);
            setWebTitle("Úprava profilu", true)
            axios.get(BASE_URL + "/user-management/user/privacy-settings", AxiosConfig()).then(res => {
                setPrivacySettings(res.data.body);
            }).catch(defaultError);

            axios.get(BASE_URL + "/user-management/user/" + cookies[userIdCookieName], AxiosConfig()).then(res => {
                res.data.body.birthDate = res.data.body.birthDate
                    ? moment(res.data.body.birthDate).format("YYYY-MM-DD")
                    : res.data.body.birthDate;
                setUserInfo(res.data.body);
            }).catch(defaultError);
        }
    }, [loadData]);



    function defaultError(err) {
        setAlertData({
            header: "Došlo k chybě.",
            content: err.response.data.errors,
            variant: CustomAlertVariants.Danger
        });
        setDisplayAlert(true);
    }

    function submitPasswordChange(e) {
        e.preventDefault();
        axios.put(BASE_URL + "/user-management/user/password", passwordChangeData, AxiosConfig()).then(res => {
            setAlertData({
                header: "Povedlo se.",
                content: res.data.body,
                variant: CustomAlertVariants.Success
            });
            setDisplayAlert(true);
        }).catch(err => {
            setAlertData({
                header: "Došlo k chybě.",
                content: err.response.data.errors,
                variant: CustomAlertVariants.Danger
            });
            setDisplayAlert(true);
        });
    }

    function handleChangePasswordForm(e) {
        const {name, value} = e.target;
        let state = {
            password: "",
            confirmPassword: "",
            oldPassword: ""
        };
        for (const [key, currValue] of Object.entries(passwordChangeData)) {
            if (key === name) {
                state[key] = value;
            } else {
                state[key] = currValue;
            }
        }
        setPasswordChangeData(state);
    }

    function uploadImage() {
        setEnableImageSend(false);
        let imageToSend = new FormData();
        imageToSend.append("image", imageData);
        axios.post(BASE_URL + "/user-management/user/image", imageToSend, AxiosConfigImage()).then(res => {
            setAlertData({
                content: res.data.body,
                variant: CustomAlertVariants.Success,
                header: "Nahrání obrázku"
            });
            setDisplayAlert(true);
        }).catch(err => {
            setAlertData({
                variant: CustomAlertVariants.Danger,
                header: "Odeslání obrázku",
                content: err.response.data.errors
            });
            setDisplayAlert(true);
            setEnableImageSend(true);
        });
    }

    function handlePrivacySettingsChange(e) {
        let {name, value} = e.target;
        setPrivacySettings({
            ...privacySettings,
            [name]: value
        });
    }

    function submitPrivacySettings() {
        axios.put(BASE_URL + "/user-management/user/privacy-settings", privacySettings, AxiosConfig()).then(res => {
            setAlertData({
                variant: CustomAlertVariants.Success,
                header: "Změna nastavení soukromí",
                content: res.data.body
            });
            setDisplayAlert(true);
            setLoadData(true);
        }).catch(err => {
            setAlertData({
                variant: CustomAlertVariants.Danger,
                header: "Změna nastavení soukromí",
                content: err.response.data.errors
            });
            setDisplayAlert(true);
        });
    }

    function deleteConfirm(password) {
        setShowModalYesNo(false);
        let config = {...AxiosConfig()};
        config.data = {password: password};
        axios.delete(BASE_URL + "/user-management/user", config).then(res => {
            setModalData({
                header: "Úspěch",
                text: res.data.body,
                type: CustomModalTypes.Info,
                buttonText: "Zpět na hlavní stránku"
            });
            setShowModal(true);
        }).catch(err => {
            setModalData({
                header: "Chyba",
                text: err.response.data.errors.map(e => e.message).join(" "),
                type: CustomModalTypes.Warning,
            });
            setShowModal(true);
        });
    }

    function submitAdditionalInfo() {
        let data = {
            bio: userInfo.bio,
            gender: userInfo.gender,
            birthDate: userInfo.birthDate
        };
        axios.put(BASE_URL + "/user-management/user", JSON.stringify(data), AxiosConfig()).then(res => {
            setAlertData({
                variant: CustomAlertVariants.Success,
                header: "Změna osobních údajů",
                content: res.data.body
            });
            setDisplayAlert(true);
            setLoadData(true);
        }).catch(defaultError);
    }

    function handleUserInfoChange(e) {
        let {name, value} = e.target;
        setUserInfo(prevState => ({
            ...prevState,
            [name]: value
        }));
    }

    const privacyOptions = [
        {value: "PRIVATE", label: "Soukromé"},
        {value: "LOGGED", label: "Pro přihlášené"},
        {value: "PUBLIC", label: "Veřejné"}
    ];
    const displayPrivacyOptions = privacyOptions.map((opt, index) => {
        return <option key={index} value={opt.value}>{opt.label}</option>
    });

    return (
        <div id='content'>
            <SEOManager description={"Úprava uživatelského profilu " + userInfo.username}
                        robots={SEO_ROBOTS.NOINDEX_NOFOLLOW}/>
            {/*Příprava pro zobrazení chyb*/}
            <CustomAlert alertData={alertData} display={displayAlert} onCloseAlert={() => setDisplayAlert(false)}/>
            <CustomModal modalData={modalData} showModal={showModal} onClose={() => {
                navigate("/");
                navigate(0);
            }}/>
            <CustomYesNoModal showModal={showModalYesNo}
                              modalData={modalDataYesNo}
                              inputData={{type: "password", label: "Zadejte heslo:"}}
                              onConfirm={deleteConfirm}
                              onCancel={() => setShowModalYesNo(false)}/>
            <Row className={"mt-3 mb-3 ms-auto me-auto"}>
                <Button variant={"outline-success"}
                        onClick={() => navigate(-1)}
                        className={"w-auto me-auto"}>Zpět na profil</Button>
                <Button variant={"danger"}
                        onClick={() => {
                            setModalDataYesNo({
                                header: "Odebrání profilu",
                                text: "Chcete opravdu odebrat profil? Tuto akci není možné vzít zpět."
                            });
                            setShowModalYesNo(true);
                        }}
                        className={"w-auto ms-auto"}>Smazat profil</Button>
            </Row>
            {/*Samotný formulář*/}
            <Accordion defaultActiveKey={['']} alwaysOpen={true}>
                {/*Změna hesla*/}
                <Accordion.Item eventKey={'0'}>
                    <Accordion.Header>Změna hesla</Accordion.Header>
                    <Accordion.Body>
                        <Form onSubmit={submitPasswordChange}>
                            <Form.Group className="mb-3" controlId="formOldPassword">
                                <Form.Label>Staré heslo</Form.Label>
                                <Form.Control type="password" placeholder="Zadejte staré heslo: "
                                              name={"oldPassword"} value={passwordChangeData.oldPassword}
                                              onChange={handleChangePasswordForm}/>
                            </Form.Group>
                            <Form.Group className="mb-3" controlId="formPassword">
                                <Form.Label>Nové heslo</Form.Label>
                                <Form.Control type="password" placeholder="Zadejte nové heslo: "
                                              name={"password"} value={passwordChangeData.password}
                                              onChange={handleChangePasswordForm}/>
                            </Form.Group>
                            <Form.Group className="mb-3" controlId="formConfirmPassword">
                                <Form.Label>Potvrďte nové heslo</Form.Label>
                                <Form.Control type="password" placeholder="Potvrďte heslo: "
                                              name={"confirmPassword"} value={passwordChangeData.confirmPassword}
                                              onChange={handleChangePasswordForm}/>
                            </Form.Group>
                            <Button variant={"primary"} type={"submit"} onSubmit={submitPasswordChange}>Uložit</Button>
                        </Form>
                    </Accordion.Body>
                </Accordion.Item>
                {/*Změna profilového obrázku*/}
                <Accordion.Item eventKey={'1'}>
                    <Accordion.Header>Změna profilového obrázku</Accordion.Header>
                    <Accordion.Body>
                        <Row>
                            <Col>
                                <span>Vyberte obrázek </span>
                                <ImageUpload displayName={true}
                                             showPreview={true}
                                             onChangeSuccess={(e) => {
                                                 setImageData(e);
                                                 setDisplayAlert(false);
                                                 setEnableImageSend(true);
                                             }}
                                             onChangeError={(err) => {
                                                 setImageData(null);
                                                 setAlertData({
                                                     variant: "danger",
                                                     header: "Chyba při výběru obrázku.",
                                                     content: err
                                                 });
                                                 setDisplayAlert(true);
                                                 setEnableImageSend(false);
                                             }}/>
                                <Button disabled={imageData === null || !enableImageSend}
                                        variant={"outline-success"}
                                        onClick={uploadImage}>Změnit obrázek</Button>
                            </Col>
                            <Col className={"text-center"}>
                                <h4>Aktuální obrázek</h4>
                                <img alt={"current-user-image"}
                                    src={USER_IMAGE_URL(userInfo.id)}
                                    style={{width:"250px", height:"250px"}}/>
                            </Col>
                        </Row>
                    </Accordion.Body>
                </Accordion.Item>
                {/*Nastavení soukromí*/}
                <Accordion.Item eventKey={'2'}>
                    <Accordion.Header>Nastavení soukromí a osobních údajů</Accordion.Header>
                    <Accordion.Body>
                        <Table responsive className={"mb-5"}>
                            <thead>
                            <tr>
                                <th className={"text-center h3"} colSpan={10}>Nastavení soukromí</th>
                            </tr>
                            </thead>
                            <tbody>
                            <tr>
                                <td>Email: {userInfo.email}</td>
                                <td>Zobrazení na profilu:</td>
                                <td>
                                    <Form.Select name={"emailPrivacy"}
                                                 value={privacySettings.emailPrivacy}
                                                 onChange={handlePrivacySettingsChange}>
                                        {displayPrivacyOptions}
                                    </Form.Select>
                                </td>
                            </tr>
                            <tr>
                                <td>Registrace: {formatDateTimeWithSettings(userInfo.createTime)}</td>
                                <td>Zobrazení na profilu:</td>
                                <td>
                                    <Form.Select name={"registrationTimePrivacy"}
                                                 value={privacySettings.registrationTimePrivacy}
                                                 onChange={handlePrivacySettingsChange}>
                                        {displayPrivacyOptions}
                                    </Form.Select>
                                </td>
                            </tr>
                            <tr>
                                <td>Komentáře článků</td>
                                <td>Zobrazení na profilu:</td>
                                <td>
                                    <Form.Select name={"commentsPrivacy"}
                                                 value={privacySettings.commentsPrivacy}
                                                 onChange={handlePrivacySettingsChange}>
                                        {displayPrivacyOptions}
                                    </Form.Select>
                                </td>
                            </tr>
                            <tr>
                                <td>Hodnocení článků</td>
                                <td>Zobrazení na profilu:</td>
                                <td>
                                    <Form.Select name={"ratingsPrivacy"}
                                                 value={privacySettings.ratingsPrivacy}
                                                 onChange={handlePrivacySettingsChange}>
                                        {displayPrivacyOptions}
                                    </Form.Select>
                                </td>
                            </tr>
                            </tbody>
                            <thead>
                            <tr>
                                <th colSpan={10} className={"text-center h3"}>Osobní údaje</th>
                            </tr>
                            </thead>
                            <tbody>
                            <tr>
                                <td>Popis</td>
                                <td className={"text-end"}>
                                    <Form.Control as={"textarea"}
                                                  name={"bio"}
                                                  value={userInfo.bio}
                                                  onChange={handleUserInfoChange}/>
                                    <Form.Text>{userInfo.bio ? userInfo.bio.length : 0}/300</Form.Text>
                                </td>
                                <td>
                                    <Form.Select name={"bioPrivacy"}
                                                 value={privacySettings.bioPrivacy}
                                                 onChange={handlePrivacySettingsChange}>
                                        {displayPrivacyOptions}
                                    </Form.Select>
                                </td>
                            </tr>
                            <tr>
                                <td>Datum narození:</td>
                                <td><Form.Control type={"date"}
                                                  name={"birthDate"}
                                                  value={userInfo.birthDate}
                                                  onChange={handleUserInfoChange}/></td>
                                <td>
                                    <Form.Select name={"birthDatePrivacy"}
                                                 value={privacySettings.birthDatePrivacy}
                                                 onChange={handlePrivacySettingsChange}>
                                        {displayPrivacyOptions}
                                    </Form.Select>
                                </td>
                            </tr>
                            <tr>
                                <td>Pohlaví:</td>
                                <td>
                                    <Form.Select name={"gender"}
                                                 value={userInfo.gender}
                                                 onChange={handleUserInfoChange}>
                                        <option value={"MEN"}>Muž</option>
                                        <option value={"FEMALE"}>Žena</option>
                                        <option value={"OTHER"}>Jiné</option>
                                    </Form.Select>
                                </td>
                                <td>
                                    <Form.Select name={"genderPrivacy"}
                                                 value={privacySettings.genderPrivacy}
                                                 onChange={handlePrivacySettingsChange}>
                                        {displayPrivacyOptions}
                                    </Form.Select>
                                </td>
                            </tr>
                            </tbody>
                            <tfoot>
                            <tr>
                                <td colSpan={1} />
                                <td colSpan={1} className={"text-center"}>
                                    <Button variant={"outline-success"}
                                            onClick={submitAdditionalInfo}>Uložit údaje</Button>
                                </td>
                                <td className={"text-center"}>
                                    <Button variant={"outline-success"}
                                            onClick={submitPrivacySettings}>
                                        Uložit soukromí
                                    </Button>
                                </td>
                            </tr>
                            </tfoot>
                        </Table>
                    </Accordion.Body>
                </Accordion.Item>
            </Accordion>
        </div>
    )
}

export default EditUser;