import React, {useEffect, useState} from "react";
import {Button, Modal, Table} from "react-bootstrap";
import {Link} from "react-router-dom";
import axios from "axios";
import {AxiosConfig, BASE_URL} from "../../components/AxiosConfig";

function ShowUserSavedArticles({show, onClose}) {

    const [savedList, setSavedList] = useState([{
        id: undefined,
        title: "Lorem ipsum článek titulek",
        articleId: 51
    }]);

    useEffect(() => {
        if (show) {
            axios.get(BASE_URL + "/user-management/user/article/saved", AxiosConfig()).then(res => {
                setSavedList(res.data.body);
            }).catch(err => {
                if(err.response.data.errors)
                    console.log(err.response.data.errors);
            });
        } else {
            setSavedList(null);
        }
    }, [show]);


    function removeArticleFromList(id){
        axios.delete(BASE_URL + "/user-management/user/article/saved/"+id, AxiosConfig()).then(res => {
            if(savedList.length > 1)
                setSavedList([...savedList.filter(item => item.id !== id)]);
            if(savedList.length === 1)
                setSavedList(null);
        }).catch(err => {
            if(err.response.data.errors)
                console.log(err.response.data.errors);
        });
    }

    return (
        <Modal show={show} onHide={onClose}>
            <Modal.Header closeButton={true}>
                <Modal.Title>Seznam uložených článků</Modal.Title>
            </Modal.Header>
            <Modal.Body>
                <Table responsive>
                    <tbody>
                    {savedList ? savedList.map((item, index) => {
                        return (
                            <tr key={index}>
                                <td>{index+1}</td>
                                <td>
                                    <Link to={"/clanek/"+item.articleId}>{item.title}</Link>
                                </td>
                                <td>
                                    <Button onClick={() => removeArticleFromList(item.id)}
                                            variant={"outline-danger"}>
                                        Odebrat
                                    </Button>
                                </td>
                            </tr>
                        )
                    }) : <tr><td colSpan={10} className={"text-center"}>Seznam je prázdný</td></tr>}
                    </tbody>
                </Table>
            </Modal.Body>
            <Modal.Footer>
                <Button variant={"outline-dark"}
                        onClick={onClose}>
                    Zavřít
                </Button>
            </Modal.Footer>
        </Modal>
    );
}

export default ShowUserSavedArticles;