import React from "react";
import {Alert} from "react-bootstrap";


/**
 * @params alertData, display, dismissible
 * @param {object} alertData Object with data for alert
 * @param {string} alertData.variant Variant of alert: success, danger
 * @param {string} alertData.header Header of alert
 * @param {string} alertData.content Text to display in alert
 * @param {Array} alertData.content Array with errors
 * @param {boolean} display
 * @param {boolean} dismissible
 * @param {$Call} onCloseAlert
 * @return Alert based on config
 * */
function customAlert({alertData, display = false, dismissible=true, onCloseAlert}) {

    return (
        <Alert hidden={!display} variant={alertData.variant} onClose={onCloseAlert} dismissible={dismissible}>
            <Alert.Heading>{alertData.header}</Alert.Heading>
            <div id="errors-p">{Array.isArray(alertData.content) ? alertData.content.map(error => ( <p key={error.key}>{error.message}</p>)) : alertData.content}</div>
        </Alert>
    );
}

export const CustomAlertVariants = {
    Danger:"danger",
    Success: "success",
    Warning: "warning",
    Info: "info"
};
export default customAlert;