import React, {useEffect, useState} from "react";
import moment from "moment";
import {Button, Col, Container, Form, Row} from "react-bootstrap";

function CustomDateRangeSelect({onConfirmRange}) {

    const [dateRange, setDateRange] = useState({
        start: moment().subtract(7, "days"),
        end: moment()
    });

    useEffect(() => {
        reload();
    }, []);

    function handleTimeChange(e) {
        let {name, value} = e.target;
        setDateRange(prevState => ({
            ...prevState,
            [name]: value
        }));
    }

    function blurTimeCheckEndStart() {
        let tempPeriod = {...dateRange};
        if (moment(tempPeriod.end).isBefore(moment(tempPeriod.start))) {
            let tempStart = tempPeriod.start;
            tempPeriod.start = tempPeriod.end;
            tempPeriod.end = tempStart;
            setDateRange(tempPeriod);
        }
    }

    function reload() {
        onConfirmRange({
            start: moment(dateRange.start),
            end: moment(dateRange.end)
        });
    }

    return (
        <Container>
            <Row className={"ms-auto me-auto w-50"}>
                <Col className={"text-center"}>
                    <Form.Label>Začátek</Form.Label>
                    <Form.Control type={"date"}
                                  name={"start"}
                                  value={moment(dateRange.start).format("yyyy-MM-DD")}
                                  onChange={handleTimeChange}
                                  onBlur={blurTimeCheckEndStart}/>
                </Col>
                <Col className={"text-center"}>
                    <Form.Label>Konec</Form.Label>
                    <Form.Control type={"date"}
                                  name={"end"}
                                  value={moment(dateRange.end).format("yyyy-MM-DD")}
                                  onChange={handleTimeChange}
                                  onBlur={blurTimeCheckEndStart}/>
                </Col>
            </Row>
            <Row className={"align-items-center justify-content-center"}>
                <Button variant={"outline-success"}
                        className={"w-25"}
                        onClick={reload}>Načíst</Button>
            </Row>
        </Container>
    )
}

export default CustomDateRangeSelect;