import {Button, Modal, Form} from "react-bootstrap";
import React from "react";

/**
 * @params modalData, showModal, onClose
 * @param {object} modalData Object with data for modal
 * @param {string} modalData.header header for modal
 * @param {string} modalData.text text to show in modal
 * @param {string} modalData.type Type for modal button
 * @param {string} [modalData.buttonText="Zavřít"] Text for button
 * @param {boolean} showModal boolean which control showing modal
 * @param {function} onClose callback to closing modal
 *
 * */
function CustomModal({modalData, showModal, onClose}) {
    if(showModal !== true){
        return null;
    }

    return (
        <Modal show={showModal} onHide={onClose}>
            <Modal.Header>{modalData.header}</Modal.Header>
            <Modal.Body>{modalData.text}</Modal.Body>
            <Modal.Footer>
                <Button variant={"outline-" + modalData.type} onClick={onClose}>
                    {modalData.buttonText ? modalData.buttonText : "Zavřít"}
                </Button>
            </Modal.Footer>
        </Modal>
    )
}
/**
 * @params modalData, showModal, onClose
 * @param {object} modalData Object with data for modal
 * @param {string} modalData.header header for modal
 * @param {string} modalData.text text to show in modal
 * @param {string} modalData.buttonConfirmType Type for modal confirm button default outline-success
 * @param {string} modalData.buttonCancelType Type for modal cancel button default outline-danger
 * @param {string} modalData.buttonConfirmText Text for confirm button default "Ano"
 * @param {string} modalData.buttonCancelText Text for cancel button default "Ne"
 *
 * @param {boolean} showModal boolean which control showing modal
 * @param {function} onCancel callback to cancel modal
 * @param {function} onConfirm callback to confirm modal
 * @param {Object} inputData Object with data for input (false to use without input)
 * @param {string} inputData.type type of input (text,password,etc)
 * @param {string} inputData.label text for input label
 *
 * */
export function CustomYesNoModal({modalData, showModal, onCancel, onConfirm, inputData}){
    if(showModal !== true){
        return null;
    }
    inputData = inputData ? inputData : false;

    function confirmButtonClick(){
        if(inputData){
            onConfirm(document.getElementById("custom-modal-input").value);
        } else {
            onConfirm();
        }
    }
    return (
        <Modal show={showModal}
               backdrop={"static"}
               keyboard={false}>
            <Modal.Header>{modalData.header}</Modal.Header>
            <Modal.Body>{modalData.text}</Modal.Body>
            {inputData
                ? <Modal.Body>
                    <Form.Label>{inputData.label}</Form.Label>
                    <Form.Control type={inputData.type} id={"custom-modal-input"}/>
                </Modal.Body>
                : ""}
            <Modal.Footer>
                <Button variant={modalData.buttonCancelType ? ("outline-" +  modalData.buttonCancelType) : CustomModalTypes.Danger}
                        onClick={onCancel}>
                    {modalData.buttonCancelText ? modalData.buttonCancelText : "Ne"}
                </Button>
                <Button variant={modalData.buttonConfirmType ? ("outline-" + modalData.buttonConfirmType) : CustomModalTypes.Success}
                        onClick={confirmButtonClick}
                >
                    {modalData.buttonConfirmText ? modalData.buttonConfirmText : "Ano"}
                </Button>
            </Modal.Footer>
        </Modal>
    )
}

export const CustomModalTypes = {
    Danger:"danger",
    Success: "success",
    Warning: "warning",
    Info: "info"
}
export default CustomModal;