import React, {useEffect, useState} from "react";
import {Pagination} from "react-bootstrap";

/**
 * @param {Integer} numOfItems Number of all items
 * @param {Integer} maxItemsPerPage Maximum items per page
 * @param {function} onChange callback when current page change param number of new page
 * @param {any} reloadDependency Reason to set page to defaultPage
 * @param {Integer} defaultPage Default page for initialization and reload
 * */
const CustomPagination = ({numOfItems, maxItemsPerPage, onChange, reloadDependency, defaultPage}) => {


    const [currentPage, setCurrentPage] = useState(1);

    useEffect(() => {
        changePage(defaultPage ? defaultPage : 1);
    }, [reloadDependency, numOfItems, maxItemsPerPage]);

    function changePage(newPage) {
        setCurrentPage(newPage);
        onChange(newPage);
    }

    return (
        <Pagination className="justify-content-center" hidden={numOfItems <= maxItemsPerPage}>
            {/*First page*/}
            <Pagination.First href={"#"}
                              onClick={() => changePage(1)}
                              hidden={currentPage === 1}/>
            {/*Current page - 1*/}
            <Pagination.Item href={"#"}
                             onClick={() => changePage((currentPage - 1))}
                             hidden={currentPage === 1}>{currentPage - 1}</Pagination.Item>
            {/*Current page*/}
            <Pagination.Item active>{currentPage}</Pagination.Item>
            {/*Current page + 1*/}
            <Pagination.Item href={"#"}
                             onClick={() => changePage((currentPage + 1))}
                             hidden={currentPage >= Math.ceil(numOfItems / maxItemsPerPage)}>{currentPage + 1}</Pagination.Item>
            {/*Last page*/}
            <Pagination.Last href={"#"}
                             onClick={() => changePage(Math.ceil(numOfItems / maxItemsPerPage))}
                             hidden={currentPage >= Math.ceil(numOfItems / maxItemsPerPage)}/>
        </Pagination>
    )
}
export default CustomPagination;