import React, {useEffect, useState} from "react";
import {Button, Form, ListGroup, Modal} from "react-bootstrap";

const ROLES = [
    "ADMIN", "USER"
]

/**
 *
 * @param {object} data
 * @param {boolean} show
 * @param {function} onClose
 * @param {function} onConfirm
 * @return {JSX.Element}
 * @constructor
 */
function EditRolesModal({data, show, onClose, onConfirm}){

    const [userRoles, setUserRoles] = useState([...data.roles]);

    useEffect(() => {
        setUserRoles([...data.roles]);
    }, [data]);

    function removeRole(role) {
        setUserRoles(userRoles.filter(item => item !== role));
    }

    function addRole(){
        let val = document.getElementById("edit-roles-modal-id-"+data.id).value;
        setUserRoles((prev) => (
            [ ...prev, val]));
    }

    return (
        <Modal show={show} onHide={onClose}>
            <Modal.Header>
                <Modal.Title>{data.header}</Modal.Title>
            </Modal.Header>
            <Modal.Body>
                <ListGroup>
                {Array.isArray(userRoles) ? userRoles.map((role, index) => {
                    return (
                        <ListGroup.Item key={index} className={"d-flex justify-content-between align-items-center"}>
                            <span>{role}</span>
                            {role === "USER" ? "" :
                                <Button variant={"outline-danger"}
                                        onClick={() => removeRole(role)}>
                                    Smazat
                                </Button>
                            }
                        </ListGroup.Item>
                    );
                }) : ""}
                    {userRoles.length === ROLES.length
                        ? ""
                        :
                        <ListGroup.Item className={"d-flex justify-content-between"}>
                            <Form.Select id={"edit-roles-modal-id-"+data.id}>
                                {ROLES.map((role, index) => {
                                    return (userRoles.includes(role)
                                        ? null: <option key={index}>{role}</option>);
                                })}
                            </Form.Select>
                            <Button className={"w-50"}
                                    onClick={addRole}
                                    variant={"outline-success"}>Přidat roli</Button>
                        </ListGroup.Item>
                    }
                </ListGroup>
            </Modal.Body>
            <Modal.Footer>
                <Button variant={"outline-danger"} onClick={onClose}>Zavřít</Button>
                <Button variant={"outline-success"} onClick={() => onConfirm(userRoles)}>Uložit</Button>
            </Modal.Footer>
        </Modal>
    );
}

export default EditRolesModal;