import React, {useState} from "react";

/**
 * @param {function} onChangeSuccess Callback when successfully change image
 * @param {function} onChangeError Callback when image change occur error
 * @param {boolean} displayName Display name of file
 * @param {boolean} showPreview Display preview
 * @param {boolean} displayHint Display hint
 * */
const ImageUpload = ({onChangeSuccess, onChangeError, displayName, showPreview, displayHint}) => {

    const maxSize = 5 * 1024 * 1024;
    const width = 250;
    const height = 250;
    const errorMaxSize = "Soubor je příliš veliký, maximální velikost je " + sizeToMB(maxSize, 0) + "MB";

    const [imageName, setImageName] = useState("");
    const [urlPreview, setUrlPreview] = useState(null);

    const handleChange = (e) => {
        if(urlPreview != null) {
            URL.revokeObjectURL(urlPreview);
            setUrlPreview(null);
        }
        if (e.target.files[0].size > maxSize) {
            onChangeError(errorMaxSize + " Velikost vašeho souboru je: " + sizeToMB(e.target.files[0].size, 1) + "MB");
            document.getElementById("image-upload-input").value = null;
            setImageName("");
        } else {
            if(displayName){
                setImageName(e.target.files[0].name);
            }
            if(showPreview){
                setUrlPreview(URL.createObjectURL(e.target.files[0]));
            }
            onChangeSuccess(e.target.files[0]);
        }
    }

    function sizeToMB(size, fixed){
        return (size / 1024 / 1024).toFixed(fixed);
    }

    return (
        <>
            {displayHint
                ? <><label className={"text-muted"}>Vhodné rozměry obrázku jsou šířka: {width}px výška: {height}px</label><br/></>
                : ""
            }
            <label style={{cursor: "pointer"}} htmlFor={"image-upload-input"}>
                <i className={"fa-solid fa-upload fa-2xl mt-2"}> </i>
            </label>
            <input style={{display: "none"}}
                   type={"file"} id={"image-upload-input"}
                   accept={"image/jpeg, image/jpg, image/png"}
                   onChange={handleChange}/>
            <br/>
            <span>Byl vybrán: {imageName}</span>
            <br/> <br/>
                <img hidden={urlPreview === null || !urlPreview}
                     src={urlPreview}
                     alt={imageName + " preview"}
                     style={{maxWidth:width, maxHeight:height}}
                     width={width}
                     height={height}/>
            <br/>
        </>
    )
}

export default ImageUpload;